from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import DirectoryView, addDirectoryViews, createDirectoryView
from Products.CMFCore.utils import getToolByName, minimalpath
from Products.CMFPortalLayout import cmfconf_globals, LayoutTemplate
from Products.ExternalMethod import ExternalMethod
from App.Common import package_home
from os import path, listdir, stat
from cStringIO import StringIO
import string

out = StringIO()

type_classes = (LayoutTemplate,)

def registerType(self, fti):
    """
        Register type defined in fti in the types tool
    """
    typestool = getToolByName(self, 'portal_types')
    if fti['id'] not in typestool.objectIds():
        cfm = apply(ContentFactoryMetadata, (), fti)
        typestool._setObject(fti['id'], cfm)
        out.write('Registered %s with the types tool\n' % fti['id'])
    else:
        out.write('Object "%s" already existed in the types tool\n' % (fti['id']))

def unregisterType(self, type):
    """
        Un-Register type in the types tool
    """
    typestool = getToolByName(self, 'portal_types')
    if type in typestool.objectIds():
        typestool._delObject(type)
        out.write('Un-Registered %s with the types tool\n' % type)
    else:
        out.write('Object "%s" did not exist in the types tool\n' % type)

def removeSkin(self, skinname):
    """
    Remove a skin from portal_skins
    """
    skinstool = getToolByName(self, 'portal_skins')
    skins = skinstool.getSkinSelections()
    for skin in skins:
        path = skinstool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        if skinname in path:
            path.remove(skinname)
            path = string.join(path, ', ')
            # addSkinSelection will replace existing skins as well.
            skinstool.addSkinSelection(skin, path)
            out.write("Removed %s from %s skin\n" % (skinname, skin))

    if skinname in skinstool.objectIds():
        skinstool._delObject(skinname)
        out.write('Removed %s from the skinstool\n' % skinname)
        
def createDefaultSetup(self):
    """
    Create a default setup for PLAC
    """
    utool = getToolByName(self, 'portal_url')
    portal = utool.getPortalObject()
    ltool = getToolByName(self, 'portal_layout')
    def_template_id = 'default_template'
    ltool.invokeFactory(id=def_template_id, type_name='Portal Template')
    out.write('Added default portal template\n')
    template = getattr(ltool, def_template_id)
    templatepath = utool.getRelativeContentURL(template)
    template.addLayoutRow()
    toprow = template.getLayoutRows()[1]
    toprowid = toprow.getId()
    template.moveLayoutRow(toprowid, direction='up')
    rows = template.getLayoutRows()
    toplayout = rows[0].getLayoutElement()
    mainlayout = rows[1].getLayoutElement()
    toplayout.addLayoutColumn()
    topcolumn = toplayout.getLayoutColumns()[0]
    topcolumn.addLayout('Breadcrumbs Element', layout_title='Breadcrumbs')
    mainlayout.addLayoutColumn()
    mainlayout.addLayoutColumn()
    movecol = mainlayout.getLayoutColumns()[1]
    mainlayout.moveLayoutColumn(movecol.getId(), direction='up')
    maincols = mainlayout.getLayoutColumns()
    leftcol = maincols[0]
    maincol = maincols[1]
    rightcol = maincols[2]
    leftcol.edit(title='Left', width='15%')
    leftcol.addLayout('Actions Box', 'User')
    leftcol.addLayout('Tool Box', 'Add content')
    usr_actions = leftcol.getLayoutElements()[0]
    usr_actions.setCategories(['user', 'global'])
    maincol.edit(title='Main', width='70%')
    rightcol.edit(title='Right', width='15%')
    rightcol.addLayout('Actions Box', 'Actions')
    obj_actions = rightcol.getLayoutElements()[0]
    obj_actions.setCategories(['object', 'folder', 'workflow'])
    obj_actions.setProvider('object')
    ltool.setLayoutTemplateFor(portal, templatepath)
    out.write('Added default template layout\n')

def install(self):
    """
    Register CMFPortalLayout classes with the necessary tools
    """

    out.write('CMF Portal Layout Product Installation on %s\n' % self.id)
    out.write('======================\n\n')

    typestool = getToolByName(self, 'portal_types')
    actionstool = getToolByName(self, 'portal_actions')
    skinstool = getToolByName(self, 'portal_skins')
    portalroot = getToolByName(self, 'portal_url').getPortalObject()
    portalcatalog = getToolByName(self, 'portal_catalog')
    wftool = getToolByName(self, 'portal_workflow')

    for c in type_classes:
        t = c.factory_type_information
        if type(t) == type(()):
            for fti in t:
                registerType(self, fti)
        else:
            registerType(self, t)

    # Create and register tools
    tools = {'portal_layout':'Layout Tool'}

    for tool in tools.keys():
        if not tool in portalroot.objectIds():
            portalroot.manage_addProduct['CMFPortalLayout'].manage_addTool(tools[tool], None)
            out.write('Created %s\n' % tools[tool])
        else:
            out.write('%s already exists\n' % tools[tool])

    ap = actionstool.listActionProviders()
    for tool in tools.keys():
        if (type(ap) is not type([]) and type(ap) is not type(())) or not tool in ap:
            actionstool.addActionProvider(tool)
            out.write('Added %s as action provider\n' % tools[tool])
        else:
            out.write('%s already existed as action provider\n' % tools[tool])
            
    wftool.setChainForPortalTypes(('Portal Template',), ())

    if not 'Language' in portalcatalog.Indexes.objectIds():
        portalcatalog.addIndex('Language', 'FieldIndex')
        out.write('Added "Language" to the portal catalog\n')
    else:
        out.write('"Language" already existed in the portal catalog\n')

    if not 'cookId' in portalroot.objectIds():
        i = ExternalMethod.ExternalMethod('cookId', 'Creates an appropriate id', 'CMFPortalLayout.utils', 'cookId')
        portalroot._setObject('cookId', i)
        out.write('Created cookId method\n')
    else:
        out.write('cookId method already existed\n')

    if not 'getNewID' in portalroot.objectIds():
        i = ExternalMethod.ExternalMethod('getNewID', 'Creates an unique id', 'CMFPortalLayout.utils', 'getNewID')
        portalroot._setObject('getNewID', i)
        out.write('Created getNewID method\n')
    else:
        out.write('getNewID method already existed\n')

    if not 'urlencode' in portalroot.objectIds():
        i = ExternalMethod.ExternalMethod('urlencode', 'URL encodes a text string', 'CMFPortalLayout.utils', 'urlencode')
        portalroot._setObject('urlencode', i)
        out.write('Created urlencode method\n')
    else:
        out.write('urlencode method already existed\n')

    if not 'gotattr' in portalroot.objectIds():
        i = ExternalMethod.ExternalMethod('gotattr', 'Use the hasattr function in Page Templates', 'CMFPortalLayout.utils', 'gotattr')
        portalroot._setObject('gotattr', i)
        out.write('Created gotattr method\n')
    else:
        out.write('gotattr method already existed\n')

    if not 'transitions' in portalroot.objectIds():
        i = ExternalMethod.ExternalMethod('transitions', 'Get transitions from portal_workflow', 'CMFPortalLayout.utils', 'getTransitions')
        portalroot._setObject('transitions', i)
        out.write('Created transitions method\n')
    else:
        out.write('transitions method already existed\n')
        
    if not 'getUniqueId' in portalroot.objectIds():
        i = ExternalMethod.ExternalMethod('getUniqueId', 'Creates an appropriate id', 'CMFPortalLayout.utils', 'getUniqueID')
        portalroot._setObject('getUniqueId', i)
        out.write('Created getUniqueId method\n')
    else:
        out.write('getUniqueId method already existed\n')
        
    if 'portal_typefilter' not in portalroot.objectIds():
        try:
            i = ExternalMethod.ExternalMethod('install_typefilter', 'Installs typefilter tool', 'CMFTypeFilter.install', 'install')
            portalroot._setObject('install_typefilter', i)
            out.write('Created install script for typefilter\n\n')
            install_txt = portalroot.install_typefilter()
            out.write(install_txt)
            out.write('\n\n')
            out.write('Type filter installed\n')
        except:
            out.write('Installation of type filter failed\n')
    else:
        out.write('Type filter already installed\n')
        
    if 'portal_configuration' not in portalroot.objectIds():
        try:
            i = ExternalMethod.ExternalMethod('install_configuration'
                                    , 'Installs configuration tool'
                                    , 'CMFConfig.install'
                                    , 'install')
            portalroot._setObject('install_configuration', i)
            out.write('Created install script for configuration\n\n')
            install_txt = portalroot.install_configuration()
            out.write(install_txt)
            out.write('\n\n')
            out.write('Configuration installed\n')
        except:
            out.write('Installation of configuration failed\n')
    else:
        out.write('Configuration already installed\n')

    # Setup the skins

    skin_name = 'portal_layout_skins'

    if skin_name not in skinstool.objectIds():
        # We need to add Filesystem Directory Views for any directories
        # in our skins/ directory.  These directories should already be
        # configured.
        fp = minimalpath("%s/zpt_skins" % package_home(cmfconf_globals))
        createDirectoryView(skinstool, fp, skin_name)
        out.write("Added %s directory view to portal_skins\n" % fp)
    else:
        out.write("%s directory view already existed in portal_skins\n" % skin_name)


    # Now we need to go through the skin configurations and insert
    # 'prosa_skins' into the configurations.  Preferably, this
    # should be right before where 'content' is placed.  Otherwise, we
    # append it to the end.
    skins = skinstool.getSkinSelections()
    for skin in skins:
        path = skinstool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        if skin_name not in path:
            try: path.insert(path.index('mmm_skins') + 1, skin_name)
            except ValueError:
                try:
                    path.insert(path.index('custom') + 1, skin_name)
                except ValueError:
                    path.append(skin_name)

            path = string.join(path, ', ')
            # addSkinSelection will replace existing skins as well.
            skinstool.addSkinSelection(skin, path)
            out.write("Added '%s' to %s 'skin'\n" % (skin_name, skin))
        else:
            out.write("Skipping %s skin, '%s' is already set up\n" % (skin, skin_name))
            
    out.write('\n')
    createDefaultSetup(self)
    return out.getvalue()

def uninstall(self):
    """
    Remove skins, types and special objects
    """
    out.write('CMF Portal Layout Un-Installation\n')
    out.write('===========================\n\n')
    portalroot = getToolByName(self, 'portal_url').getPortalObject()
    for c in type_classes:
        t = c.factory_type_information
        if type(t) == type(()):
            for fti in t:
                unregisterType(self, fti['id'])
        else:
            unregisterType(self, t['id'])

    removeSkin(self, 'portal_layout_skins')

    return out.getvalue()
