import zLOG
import sys, traceback
from cStringIO import StringIO
from Products.CMFCore.utils import getToolByName
from Products.ExternalMethod import ExternalMethod

out = StringIO()

def beforeUpgrade(self):
    """
    We need to save some data about how the old layout is
    We will save the layout from the portal root and the local columns
    We will also save the data for the layout types
    """
    portal_url = getToolByName(self, 'portal_url')
    portal = portal_url.getPortalObject()
    layouttool = getToolByName(self, 'portal_layout')
    pagesetup = layouttool.getLayoutPropertiesFor(portal)
    try:
        maincol = layouttool.getMainColumn(portal)
    except:
        try:
            maincol = layouttool.getMainColumn()
        except:
            maincol = None
    topbar = layouttool.getLayoutTopbarFor(portal, aquire=0)
    columns = layouttool.getLayoutColumnsFor(portal)
    maincolidx = 0
    idx = 0
    for col in columns:
        if col == maincol:
           maincolidx = idx
           break
        idx = idx + 1
    if not maincolidx:
        maincolidx = 1
        
    out.write('Main column is located as number %d column\n' % (idx+1))
        
    conffile = open('oldPortalLayoutSetup.txt', 'w')
    
    settingstxt = 'settings;width=%s;align=%s;cellspacing=%s;cellpadding=%s;maincolumnidx=%d\n' % \
        (pagesetup['width'], pagesetup['align'], pagesetup['cellspacing'], pagesetup['cellpadding'], maincolidx)
    conffile.write(settingstxt)
    out.write('Saved page layout settings\n\n')
    
    
    #Topbar stuff
    topbarcolumns = []
    topbarelems = layouttool.listLayoutObjectsFor(topbar, 'topbar')
    column = {}
    column['width'] = topbar.getProperty('width')
    column['title'] = topbar.title
    saveelems = []
    if topbarelems:
        for element in topbarelems:
            saveelems.append(parseElementData(element))
    column['elements'] = saveelems
    topbarcolumns.append(column)
    rowtxt = "row;%s\n" % str(topbarcolumns)
    conffile.write(rowtxt)
    out.write('Saved topbar settings\n')
    
    # Column stuff
    pagecolumns = []
    for col in columns:
        column = {}
        column['width'] = col.getProperty('width')
        column['title'] = col.title
        saveelems = []
        colelements = layouttool.listLayoutObjectsFor(col)
        if colelements:
            for element in colelements:
                saveelems.append(parseElementData(element))
        column['elements'] = saveelems
        pagecolumns.append(column)
    rowtxt = "row;%s\n" % str(pagecolumns)
    conffile.write(rowtxt)
    out.write('Saved columns settings\n\n')
    
    out.write('Fething local column data\n')
    localcols = {}
    localcols = getLocalColumns(layouttool, portal_url, portal, localcols)
    out.write('\n')
    lcolstxt = "locals;%s\n" % str(localcols)
    conffile.write(lcolstxt)
    conffile.close()
    out.write('Saved layout columns and closed file\n')
    out.write('Now you can upgrade CMFPortalLayout and restart Zope\n')
    out.write("Afterwards you must create the 'doUpgrade' External Method in order to upgrade your portal layout")
    return out.getvalue()

def getLocalColumns(ltool, utool, folder, localcols):
    """
    Find the local columns (if any)
    """
    for obj in folder.contentValues():
        if obj.isPrincipiaFolderish and obj.meta_type != 'CMF Collector': #We need to skip the tracker
            zLOG.LOG('get Local Columns', zLOG.INFO, 'Obj path is %s' % obj.absolute_url(relative=1))
            out.write('Finding local columns for %s\n' % obj.absolute_url(relative=1))
            locals = extractLocalColumns(ltool, obj)
            if locals:
                objpath = utool.getRelativeContentURL(obj)
                localcols[objpath] = locals
                out.write('Found local columns\n')
            else:
                out.write('No local columns found\n')
            localcols = getLocalColumns(ltool, utool, obj, localcols)
    return localcols

def extractLocalColumns(ltool, obj):
    """
    Extract the local columns for the folder
    """
    locals = ltool.getLocalLayoutColumnsFor(obj, aquire=0)
    columns = []
    if locals:
        for column in locals:
            coldict = {}
            coldict['title'] = column.getProperty('title')
            coldict['width'] = column.getProperty('width')
            layouts = []
            for layout in ltool.listLayoutObjectsFor(column):
                layouts.append(parseElementData(layout))
            coldict['layouts'] = layouts
            columns.append(coldict)
    return columns

def parseElementData(element, datadict=None, action='read'):
    """
    Determines the metatype of the element and calls the correct converter
    """
    mtype = element.meta_type
    data = None
    if mtype == 'About Box' or mtype == 'Info Box':
        if action == 'read':
            data = getInfoBoxData(element)
        elif action == 'write':
            writeInfoBoxData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'Actions Menu' or mtype == 'Actions Box':
        if action == 'read':
            data = getActionBoxData(element)
        elif action == 'write':
            writeActionBoxData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'Breadcrumbs Box' or mtype == 'Breadcrumbs Element':
        if action == 'read':
            data = getBreadcrumbBoxData(element)
        elif action == 'write':
            writeBreadcrumbBoxData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'Calendar Box':
        if action == 'read':
            data = getCalendarBoxData(element)
        elif action == 'write':
            writeCalendarBoxData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'Content Menu' or mtype == 'Content Element':
        if action == 'read':
            data = getContentMenuData(element)
        elif action == 'write':
            writeContentMenuData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'Image Menu' or mtype == 'Image Element':
        if action == 'read':
            data = getImageMenuData(element)
        elif action == 'write':
            writeImageMenuData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'Interval List' or mtype == 'Interval Box':
        if action == 'read':
            data = getIntervalListData(element)
        elif action == 'write':
            writeIntervalListData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'Language Box' or mtype == 'Language Element':
        if action == 'read':
            data = getLanguageBoxData(element)
        elif action == 'write':
            writeLanguageBoxData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'Layout Box' or mtype == 'Layout Element':
        if action == 'read':
            data = getLayoutBoxData(element)
        elif action == 'write':
            writeLayoutBoxData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'Link Menu' or mtype == 'Link Box':
        if action == 'read':
            data = getLinkBoxData(element)
        elif action == 'write':
            writeLinkBoxData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'List Menu' or mtype == 'List Box':
        if action == 'read':
            data = getListMenuData(element)
        elif action == 'write':
            writeListMenuData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'Login Box':
        if action == 'read':
            data = getLoginBoxData(element)
        elif action == 'write':
            writeLoginBoxData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'Ordered Menu' or mtype == 'Ordered Box':
        if action == 'read':
            data = getOrderedMenuData(element)
        elif action == 'write':
            writeOrderedMenuData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'PPV Time Box':
        if action == 'read':
            data = getPPVTimeBoxData(element)
        elif action == 'write':
            writePPVTimeBoxData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'RSS Menu' or mtype == 'RSS Box':
        if action == 'read':
            data = getRSSMenuData(element)
        elif action == 'write':
            writeRSSMenuData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'Search Box' or mtype == 'Search Element':
        if action == 'read':
            data = getSearchBoxData(element)
        elif action == 'write':
            writeSearchBoxData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'Subject Search Menu' or mtype == 'Subject Search Box':
        if action == 'read':
            data = getSubjectSearchData(element)
        elif action == 'write':
            writeSubjectSearchData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'Tool Box':
        if action == 'read':
            data = getToolBoxData(element)
        elif action == 'write':
            writeToolBoxData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    if mtype == 'User Group List' or mtype == 'User Group Box':
        if action == 'read':
            data = getUserGroupsListData(element)
        elif action == 'write':
            writeUserGroupsListData(element, datadict)
        else:
            raise AttributeError, 'Unknown action %s' % action
    return data

def getListMenuData(listmenu):
    """
    Return the data for the listmenu as a dictionary
    """
    data = {}
    data['metatype'] = 'List Box'
    data['title'] = listmenu.title
    data['format'] = listmenu.getFormat()
    data['itemCount'] = listmenu.getItemCount()
    data['subjects'] = listmenu.getSubjects()
    try:
        data['ownSubjects'] = listmenu.getOwnSubjects()
    except:
        pass
    data['types'] = listmenu.getTypes()
    data['search'] = listmenu.getSearch()
    data['sortcriteria'] = listmenu.getSortCriteria()
    data['sortorder'] = listmenu.getCatalogSortOrder()
    data['selectSubjects'] = listmenu.getSubjectSelector() #The subjects that shoud be listed in a subject selector
    data['useSelector'] = listmenu.useSubjectSelector()
    data['language'] = listmenu.getLanguage()
    data['useLanguage'] = listmenu.useLanguageInSearch()
    data['showUpFor'] = listmenu.getShowupFor()
    data['useBrowseButtons'] = listmenu.useBrowseButtons()
    try:
        data['userOrGroupSearch'] = listmenu.getUserOrGroupSearch()
    except:
        pass
    return data

def writeListMenuData(listmenu, data):
    """
    Write the data for the listmenu
    """
    listmenu.setTitle(data['title'])
    try:
        listmenu.setFormat(data['format'])
    except:
        zLOG.LOG('write List Menu', zLOG.ERROR, 'Error: List: %s\n%s - %s\n%s' % \
            (listmenu.absolute_url(relative=1)
           , sys.exc_info()[0]
           , sys.exc_info()[1]
           , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
    try:
        listmenu.setItemCount(data['itemCount'])
    except:
        zLOG.LOG('write List Menu', zLOG.ERROR, 'Error: %s - %s\n%s' % \
            (sys.exc_info()[0]
           , sys.exc_info()[1]
           , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
    try:
        listmenu.setSubjects(data['subjects'])
    except:
        zLOG.LOG('write List Menu', zLOG.ERROR, 'Error: %s - %s\n%s' % \
            (sys.exc_info()[0]
           , sys.exc_info()[1]
           , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
    try:
        listmenu.setOwnSubjects(data['ownSubjects'])
    except:
        zLOG.LOG('write List Menu', zLOG.ERROR, 'Error: %s - %s\n%s' % \
            (sys.exc_info()[0]
           , sys.exc_info()[1]
           , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
    try:
        listmenu.setTypes(data['types'])
    except:
        zLOG.LOG('write List Menu', zLOG.ERROR, 'Error: %s - %s\n%s' % \
            (sys.exc_info()[0]
           , sys.exc_info()[1]
           , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
    try:
        listmenu.setSearch(data['search'])
    except:
        zLOG.LOG('write List Menu', zLOG.ERROR, 'Error: %s - %s\n%s' % \
            (sys.exc_info()[0]
           , sys.exc_info()[1]
           , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
    try:
        listmenu.setSortCriteria(data['sortcriteria'])
    except:
        zLOG.LOG('write List Menu', zLOG.ERROR, 'Error: %s - %s\n%s' % \
            (sys.exc_info()[0]
           , sys.exc_info()[1]
           , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
    try:
        listmenu.setCatalogSortOrder(data['sortorder'])
    except:
        zLOG.LOG('write List Menu', zLOG.ERROR, 'Error: %s - %s\n%s' % \
            (sys.exc_info()[0]
           , sys.exc_info()[1]
           , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
    try:
        listmenu.setSubjectSelector(data['selectSubjects']) #The subjects that shoud be listed in a subject selector
    except:
        zLOG.LOG('write List Menu', zLOG.ERROR, 'Error: %s - %s\n%s' % \
            (sys.exc_info()[0]
           , sys.exc_info()[1]
           , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
    try:
        if data['useSelector']:
            useSelector = 1
        else:
            useSelector = None
    except:
        useSelector = None
    listmenu.setUseSelector(useSelector)
    try:
        listmenu.setLanguage(data['language'])
    except:
        zLOG.LOG('write List Menu', zLOG.ERROR, 'Error: %s - %s\n%s' % \
            (sys.exc_info()[0]
           , sys.exc_info()[1]
           , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
    try:
        listmenu.setSearchLanguage(data['useLanguage'])
    except:
        zLOG.LOG('write List Menu', zLOG.ERROR, 'Error: %s - %s\n%s' % \
            (sys.exc_info()[0]
           , sys.exc_info()[1]
           , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
    try:
        listmenu.setShowupFor(data['showUpFor'])
    except:
        zLOG.LOG('write List Menu', zLOG.ERROR, 'Error: %s - %s\n%s' % \
            (sys.exc_info()[0]
           , sys.exc_info()[1]
           , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
    try:
        listmenu.setUseBrowseButtons(data['useBrowseButtons'])
    except:
        zLOG.LOG('write List Menu', zLOG.ERROR, 'Error: %s - %s\n%s' % \
            (sys.exc_info()[0]
           , sys.exc_info()[1]
           , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
    try:
        listmenu.setUserOrGroupSearch(data['userOrGroupSearch'])
    except:
        zLOG.LOG('write List Menu', zLOG.ERROR, 'Error: %s - %s\n%s' % \
            (sys.exc_info()[0]
           , sys.exc_info()[1]
           , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))

def getInfoBoxData(infobox):
    """
    Return the data for the infobox as a dictionary
    """
    data = {}
    data['metatype'] = 'Info Box'
    data['title'] = infobox.title
    data['format'] = infobox.getFormat()
    data['language'] = infobox.getLanguage()
    data['showUpFor'] = infobox.getShowupFor()
    return data

def writeInfoBoxData(infobox, data):
    """
    Write the data for the infobox
    """
    infobox.setTitle(data['title'])
    infobox.setFormat(data['format'])
    infobox.setLanguage(data['language'])
    infobox.setShowupFor(data['showUpFor'])

def getActionBoxData(actionbox):
    """
    Return the data for the action box as a dictionary
    """
    data = {}
    data['metatype'] = 'Actions Box'
    data['title'] = actionbox.title
    data['categories'] = actionbox.getCategories()
    data['style'] = actionbox.getStyle()
    data['provider'] = actionbox.getProvider()
    return data

def writeActionBoxData(actionbox, data):
    """
    Write the data for the action box
    """
    actionbox.setTitle(data['title'])
    actionbox.setCategories(data['categories'])
    actionbox.setStyle(data['style'])
    actionbox.setProvider(data['provider'])

def getBreadcrumbBoxData(breadcrumbbox):
    """
    Return the data for the breadcrumb box as a dictionary
    """
    data = {}
    data['metatype'] = 'Breadcrumbs Element'
    data['title'] = breadcrumbbox.title
    data['includeRoot'] = breadcrumbbox.includeRoot()
    data['style'] = breadcrumbbox.getStyle()
    data['addToFavorites'] = breadcrumbbox.addToFavorites()
    return data

def writeBreadcrumbBoxData(breadcrumbbox, data):
    """
    Write the data for the breadcrumb box
    """
    breadcrumbbox.setTitle(data['title'])
    if data['includeRoot']:
        incRoot = 1
    else:
        incRoot = None
    breadcrumbbox.setIncludeRoot(incRoot)
    breadcrumbbox.setStyle(data['style'])
    if data['addToFavorites']:
        addFav = 1
    else:
        addFav = None
    breadcrumbbox.setAddToFavorites(addFav)

def getCalendarBoxData(calendar):
    """
    Return the data for the calendar box as a dictionary
    """
    data = {}
    data['metatype'] = calendar.meta_type
    data['title'] = calendar.title
    data['format'] = calendar.getFormat()
    data['itemCount'] = calendar.getItemCount()
    data['subjects'] = calendar.getSubjects()
    data['types'] = calendar.getTypes()
    data['search'] = calendar.getSearch()
    data['sortcriteria'] = calendar.getSortCriteria()
    data['sortorder'] = calendar.getCatalogSortOrder()
    data['language'] = calendar.getLanguage()
    data['useLanguage'] = calendar.useLanguageInSearch()
    data['userOrGroupSearch'] = calendar.getUserOrGroupSearch()
    return data

def writeCalendarBoxData(calendar, data):
    """
    Write the data for the calendar box
    """
    calendar.setTitle(data['title'])
    calendar.setFormat(data['format'])
    calendar.setItemCount(data['itemCount'])
    calendar.setSubjects(data['subjects'])
    calendar.setTypes(data['types'])
    calendar.setSearch(data['search'])
    calendar.setSortCriteria(data['sortcriteria'])
    calendar.setCatalogSortOrder(data['sortorder'])
    calendar.setLanguage(data['language'])
    calendar.setSearchLanguage(data['useLanguage'])
    calendar.setUserOrGroupSearch(data['userOrGroupSearch'])

def getContentMenuData(contentmenu):
    """
    Return the data for the content menu as a dictionary
    """
    data = {}
    data['metatype'] = 'Content Element'
    data['title'] = contentmenu.title
    data['language'] = contentmenu.getLanguage()
    data['renderTitle'] = contentmenu.renderTitle()
    data['renderHeader'] = contentmenu.renderHeader()
    data['contentPaths'] = contentmenu.getContentPaths()
    return data

def writeContentMenuData(contentmenu, data):
    """
    Write the data for the content menu
    """
    contentmenu.setTitle(data['title'])
    contentmenu.setLanguage(data['language'])
    contentmenu.setRenderTitle(data['renderTitle'])
    contentmenu.setRenderHeader(data['renderHeader'])
    for path in data['contentPaths']:
        contentmenu.setContent(contentpath=path)

def getImageMenuData(imagemenu):
    """
    Return the data for the image menu as a dictionary
    """
    data = {}
    data['metatype'] = 'Image Element'
    data['title'] = imagemenu.title
    data['content'] = imagemenu.getContentPath()
    data['renderTitle'] = imagemenu.renderTitle()
    data['renderHeader'] = imagemenu.renderHeader()
    data['link'] = imagemenu.getLink()
    return data

def writeImageMenuData(imagemenu, data):
    """
    Write the data for the image menu
    """
    imagemenu.setTitle(data['title'])
    imagemenu.setContent(data['content'])
    imagemenu.setRenderTitle(data['renderTitle'])
    imagemenu.setRenderHeader(data['renderHeader'])
    imagemenu.setLink(data['link'])

def getIntervalListData(intervallist):
    """
    Return the data for the interval list as a dictionary
    """
    data = {}
    data['metatype'] = 'Interval Box'
    data['title'] = intervallist.title
    data['format'] = intervallist.getFormat()
    data['itemCount'] = intervallist.getItemCount()
    data['subjects'] = intervallist.getSubjects()
    data['types'] = intervallist.getTypes()
    data['search'] = intervallist.getSearch()
    data['sortcriteria'] = intervallist.getSortCriteria()
    data['sortorder'] = intervallist.getCatalogSortOrder()
    data['selectSubjects'] = intervallist.getSubjectSelector() #The subjects that shoud be listed in a subject selector
    data['useSelector'] = intervallist.useSubjectSelector()
    data['language'] = intervallist.getLanguage()
    data['useLanguage'] = intervallist.useLanguageInSearch()
    data['showUpFor'] = intervallist.getShowupFor()
    data['userOrGroupSearch'] = intervallist.getUserOrGroupSearch()
    data['startCriteria'] = intervallist.getStartCriteria()
    data['endCriteria'] = intervallist.getEndCriteria()
    return data

def writeIntervalListData(intervallist, data):
    """
    Write the data for the interval list
    """
    intervallist.setTitle(data['title'])
    intervallist.setFormat(data['format'])
    intervallist.setItemCount(data['itemCount'])
    intervallist.setSubjects(data['subjects'])
    intervallist.setTypes(data['types'])
    intervallist.setSearch(data['search'])
    intervallist.setSortCriteria(data['sortcriteria'])
    intervallist.setCatalogSortOrder(data['sortorder'])
    intervallist.setSubjectSelector(data['selectSubjects']) #The subjects that shoud be listed in a subject selector
    if data['useSelector']:
        useSelector = 1
    else:
        useSelector = None
    intervallist.setUseSelector(useSelector)
    intervallist.setLanguage(data['language'])
    intervallist.setSearchLanguage(data['useLanguage'])
    intervallist.setShowupFor(data['showUpFor'])
    intervallist.setUserOrGroupSearch(data['userOrGroupSearch'])
    intervallist.setStartCriteria(data['startCriteria'])
    intervallist.setEndCriteria(data['endCriteria'])

def getLanguageBoxData(languagebox):
    """
    Return the data for the language box as a dictionary
    """
    data = {}
    data['metatype'] = 'Language Element'
    data['title'] = languagebox.title
    data['language'] = languagebox.getLanguage()
    data['renderTitle'] = languagebox.renderTitle()
    return data

def writeLanguageBoxData(languagebox, data):
    """
    Write the data for the language box
    """
    languagebox.setTitle(data['title'])
    languagebox.setLanguage(data['language'])
    languagebox.setRenderTitle(data['renderTitle'])

def getLinkBoxData(linkbox):
    """
    Return the data for the link box as a dictionary
    """
    data = {}
    data['metatype'] = 'Link Box'
    data['title'] = linkbox.title
    data['format'] = linkbox.getFormat()
    data['style'] = linkbox.getStyle()
    data['showUpFor'] = linkbox.getShowupFor()
    data['language'] = linkbox.getLanguage()
    data['links'] = linkbox.getLinks()
    return data

def writeLinkBoxData(linkbox, data):
    """
    Write the data for the link box
    """
    linkbox.setTitle(data['title'])
    linkbox.setFormat(data['format'])
    linkbox.setStyle(data['style'])
    linkbox.setShowupFor(data['showUpFor'])
    linkbox.setLanguage(data['language'])
    linkbox.initializeLinks(data['links'])

def getLoginBoxData(loginbox):
    """
    Return the data for the login box as a dictionary
    """
    data = {}
    data['metatype'] = 'Login Box'
    data['title'] = loginbox.title
    data['style'] = loginbox.getStyle()
    data['renderTitle'] = loginbox.renderTitle()
    data['language'] = loginbox.getLanguage()
    return data

def writeLoginBoxData(loginbox, data):
    """
    Write the data for the login box
    """
    loginbox.setTitle(data['title'])
    loginbox.setStyle(data['style'])
    loginbox.setRenderTitle(data['renderTitle'])
    loginbox.setLanguage(data['language'])

def getOrderedMenuData(orderedmenu):
    """
    Return the data for the ordered menu as a dictionary
    """
    data = {}
    data['metatype'] = 'Ordered Box'
    data['title'] = orderedmenu.title
    data['format'] = orderedmenu.getFormat()
    data['renderTitle'] = orderedmenu.renderTitle()
    data['objectList'] = orderedmenu.getObjectList() #Dictionary containing the information
    return data

def writeOrderedMenuData(orderedmenu, data):
    """
    Write the data for the ordered menu
    """
    orderedmenu.setTitle(data['title'])
    orderedmenu.setFormat(data['format'])
    orderedmenu.setRenderTitle(data['renderTitle'])
    selids = []
    for objdict in data['objectList']:
       if objdict['isListed']:
           selids.append(objdict['id'])
    orderedmenu.setSelectedObjects(selids)

def getPPVTimeBoxData(ppvbox):
    """
    Return the data for the ppv time box as a dictionary
    """
    data = {}
    data['metatype'] = ppvbox.meta_type
    data['title'] = ppvbox.title
    data['language'] = ppvbox.getLanguage()
    data['renderTitle'] = ppvbox.renderTitle()
    data['style'] = ppvbox.getStyle()
    return data

def writePPVTimeBoxData(ppvbox, data):
    """
    Write the data for the ppv time box
    """
    ppvbox.setTitle(data['title'])
    ppvbox.setLanguage(data['language'])
    ppvbox.setRenderTitle(data['renderTitle'])
    ppvbox.setStyle(data['style'])

def getRSSMenuData(rssmenu):
    """
    Return the data for the rss menu as a dictionary
    """
    data = {}
    data['metatype'] = 'RSS Box'
    data['title'] = rssmenu.title
    data['itemCount'] = rssmenu.getItemCount()
    data['url'] = rssmenu.getURL()
    data['format'] = rssmenu.getFormat()
    data['renderTitle'] = rssmenu.renderTitle()
    return data

def writeRSSMenuData(rssmenu, data):
    """
    Write the data for the rss menu
    """
    rssmenu.setTitle(data['title'])
    rssmenu.setItemCount(data['itemCount'])
    rssmenu.setURL(data['url'])
    rssmenu.setFormat(data['format'])
    rssmenu.setRenderTitle(data['renderTitle'])
    rssmenu.update()

def getSearchBoxData(searchbox):
    """
    Return the data for the search time box as a dictionary
    """
    data = {}
    data['metatype'] = 'Search Element'
    data['title'] = searchbox.title
    data['language'] = searchbox.getLanguage()
    data['renderTitle'] = searchbox.renderTitle()
    data['style'] = searchbox.getStyle()
    return data

def writeSearchBoxData(searchbox, data):
    """
    Write the data for the search time box
    """
    searchbox.setTitle(data['title'])
    searchbox.setLanguage(data['language'])
    searchbox.setRenderTitle(data['renderTitle'])
    searchbox.setStyle(data['style'])

def getSpacerElementData(spacerelement):
    """
    Return the data for the ppv time box as a dictionary
    """
    data = {}
    data['metatype'] = spacerelement.meta_type
    data['title'] = spacerelement.title
    data['height'] = spacerelement.getHeight()
    data['width'] = spacerelement.getWidth()
    return data

def writeSpacerElementData(spacerelement, data):
    """
    Write the data for the ppv time box
    """
    spacerelement.setTitle(data['title'])
    spacerelement.setHeight(data['height'])
    spacerelement.setWidth(data['width'])

def getSubjectSearchData(subjectsearch):
    """
    Return the data for the subject search as a dictionary
    """
    data = {}
    data['metatype'] = 'Subject Search Box'
    data['title'] = subjectsearch.title
    data['itemCount'] = subjectsearch.getItemCount()
    data['types'] = subjectsearch.getTypes()
    data['search'] = subjectsearch.getSearch()
    data['style'] = subjectsearch.getStyle()
    data['renderTitle'] = subjectsearch.renderTitle()
    return data

def writeSubjectSearchData(subjectsearch, data):
    """
    Write the data for the subject search
    """
    subjectsearch.setTitle(data['title'])
    subjectsearch.setItemCount(data['itemCount'])
    subjectsearch.setTypes(data['types'])
    subjectsearch.setSearch(data['search'])
    subjectsearch.setStyle(data['style'])
    subjectsearch.setRenderTitle(data['renderTitle'])

def getToolBoxData(toolbox):
    """
    Return the data for the tool box as a dictionary
    """
    data = {}
    data['metatype'] = toolbox.meta_type
    data['title'] = toolbox.title
    data['language'] = toolbox.getLanguage()
    data['renderTitle'] = toolbox.renderTitle()
    data['style'] = toolbox.getStyle()
    return data

def writeToolBoxData(toolbox, data):
    """
    Write the data for the tool box
    """
    toolbox.setTitle(data['title'])
    toolbox.setLanguage(data['language'])
    toolbox.setRenderTitle(data['renderTitle'])
    toolbox.setStyle(data['style'])

def getUserGroupsListData(usergrouplist):
    """
    Return the data for the user group list as a dictionary
    """
    data = {}
    data['metatype'] = 'User Group Box'
    data['title'] = usergrouplist.title
    data['itemCount'] = usergrouplist.getItemCount()
    data['format'] = usergrouplist.getFormat()
    data['useBrowseButtons'] = usergrouplist.useBrowseButtons()
    data['groupid'] = usergrouplist.getGroupId()
    return data

def writeUserGroupsListData(usergrouplist, data):
    """
    Write the data for the user group list
    """
    usergrouplist.setTitle(data['title'])
    usergrouplist.setItemCount(data['itemCount'])
    usergrouplist.setFormat(data['format'])
    usergrouplist.setUseBrowseButtons(data['useBrowseButtons'])
    usergrouplist.setGroupId(data['groupid'])
    return data

def getLayoutBoxData(layoutbox):
    """
    Return the data for the layout box as a dictionary
    """
    data =  {}
    data['metatype'] = 'Layout Element'
    data['title'] = layoutbox.title
    cols = []
    for column in layoutbox.listColumns():
        col = {}
        colwidth = column.getProperty('width')
        coltitle = column.getProperty('title')
        col['width'] = colwidth
        col['title'] = coltitle
        layouts = []
        if layoutbox.listLayoutsForColumn(column):
            for layout in layoutbox.listLayoutsForColumn(column):
                layouts.append(parseElementData(layout))
        col['layouts'] = layouts
        cols.append(col)
    data['columns'] = cols
    return data

def writeLayoutBoxData(layoutbox, data):
    """
    Write the data for the layout box
    """
    layoutbox.setTitle(data['title'])
    numcols = len(data['columns'])
    i = 0
    while i < numcols:
        layoutbox.addLayoutColumn()
        i = i + 1
        
    i = 0
    boxcolumns = layoutbox.getLayoutColumns()
    for column in boxcolumns:
        coldict = data['columns'][i]
        column.edit(title=coldict['title'], width=coldict['width'])
        for layout in coldict['layouts']:
            ltype = layout['metatype']
            column.addLayout(ltype)
        lidx = 0
        for layout in column.getLayoutElements():
            ldata = coldict['layouts'][lidx]
            parseElementData(layout, datadict=ldata, action='write')
            lidx = lidx + 1
        i = i + 1
        
def doUpgrade(self):
    """
    You must have saved some data before installing the new version
    of CMFPortalLayout on your filesystem.
    We will use the backup file created from the 'beforeUpgrade' method
    and install the new version of CMFPortalLayout in your Zope system.
    """
    out.write('Upgrading your Portal Layout installation\n\n')
    portal_url = getToolByName(self, 'portal_url')
    portal = portal_url.getPortalObject()
    deleteOldLayout(portal)
    maincolumnidx = None
    mainrowidx = 1
    install_id = 'install_plac'
    if install_id in portal.objectIds():
        portal.manage_delObjects(install_id)
    i = ExternalMethod.ExternalMethod(install_id, 'Install PLAC', 'CMFPortalLayout.install', 'install')
    portal._setObject(install_id, i)
    install_txt = portal.install_plac()
    out.write(install_txt)
    out.write('\n\n')
    try:
        layouttool = getToolByName(self, 'portal_layout')
    except:
        pass
    templateid = 'converted_layout'
    portal.invokeFactory(id=templateid, type_name='Portal Template')
    def_template = getattr(portal, templateid)
    out.write('Created portal template\n')
    
    conffile = open('oldPortalLayoutSetup.txt', 'r')
    
    rows = []
    locals = None
    for line in conffile.readlines():
        datatype, data = line.split(';', 1)
        if datatype == 'settings':
            settingspair = data.split(';')
            settings = {}
            for pair in settingspair:
                key, value = pair.split('=')
                settings[key] = value
            cellpadding = settings['cellpadding']
            cellspacing = settings['cellspacing']
            if not cellpadding:
                cellpadding = 0
            if not cellspacing:
                cellspacing = 0
            def_template.edit(pagewidth=settings['width']
                            , align=settings['align']
                            , cellpadding=cellpadding
                            , cellspacing=cellspacing)
            maincolumnidx = int(settings['maincolumnidx'])
        elif datatype == 'row':
            rows.append(data)
        elif datatype == 'locals':
            locals = eval(data)
        out.write('\n')
        
    rowcount = 0
    zLOG.LOG('Parsing rows', zLOG.INFO, 'Main row idx %d' % mainrowidx)
    for row in rows:
        zLOG.LOG('Parsing rows', zLOG.INFO, 'Row count %d' % rowcount)
        if rowcount != mainrowidx:
            def_template.addLayoutRow()
            rowidx = len(def_template.getLayoutRows()) - 1
            zLOG.LOG('Parsing rows', zLOG.INFO, 'Row idx %d' % rowidx)
            portalrow = def_template.getLayoutRows()[rowidx]
            isMainRow = 0
            out.write('Not main row - creating a new one\n')
            if rowcount < mainrowidx and rowidx >= mainrowidx:
                def_template.moveLayoutRow(portalrow.getId(), direction='up')
                out.write('Moving row up before main row\n')
            isMainRow = 0
        else:
            isMainRow = 1
        portalrow = def_template.getLayoutRows()[rowcount]
        rowlayout = portalrow.getLayoutElement()
        tmpelems = rowlayout.getLayoutColumns()
        zLOG.LOG('Parsing rows', zLOG.INFO, 'Elements in row is:\n%s' % str(tmpelems))
        columncount = 0
        if type(row) is type(''):
            row = eval(row)
        for columndata in row:
            if columncount == maincolumnidx and isMainRow:
                column = rowlayout.getMainColumn()
                out.write('Found main column - not creating a new one\n')
            else:
                rowlayout.addLayoutColumn()
                columnidx = len(rowlayout.getLayoutColumns())-1
                column = rowlayout.getLayoutColumns()[columnidx]
                out.write('Not main column - creating a new one\n')
                if columncount < maincolumnidx and isMainRow:
                    rowlayout.moveLayoutColumn(column.getId(), direction='up')
                    out.write('Moving column up before main column\n')
            column.edit(title=columndata['title']
                      , width=columndata['width'])
            if columncount == maincolumnidx and isMainRow:
                layouttool.addLocalLayoutColumn(portal)
                column = layouttool.getLocalLayoutColumns(portal, aquire=0)[0]
                out.write('Found main column - adding local columns for portal\n')
            elementcount = 0
            for elementdata in columndata['elements']:
                mtype = elementdata['metatype']
                column.addLayout(mtype)
                elcols = column.getLayoutElements()
                element = column.getLayoutElements()[elementcount]
                parseElementData(element
                                , datadict=elementdata
                                , action='write')
                elementcount = elementcount + 1
                out.write('Added %s to column %d in row %d\n' % \
                    (mtype, columncount, rowcount))
            columncount = columncount + 1
        rowcount = rowcount + 1
        out.write('\n')

    if locals:
        out.write('Adding local layout\n')
        for lpath in locals.keys():
            try:
                obj = portal.restrictedTraverse(lpath)
            except:
                obj = None
            if obj:
                out.write('Setting up layout for %s\n' % \
                          obj.absolute_url(relative=1))
                columns = locals[lpath]
                colidx = 0
                for coldict in columns:
                    layouttool.addLocalLayoutColumn(obj)
                    column = layouttool.getLocalLayoutColumns(obj, aquire=0)[colidx]
                    column.edit(title=coldict['title'], width=coldict['width'])
                    layouts = coldict['layouts']
                    layoutidx = 0
                    for layoutdata in layouts:
                        ltype = layoutdata['metatype']
                        column.addLayout(ltype)
                        layout = column.getLayoutElements()[layoutidx]
                        parseElementData(layout, layoutdata, action='write')
                        layoutidx = layoutidx + 1
                    colidx = colidx + 1
            else:
                out.write('Object %s not found\n' % lpath)
                
        out.write('\n')
    
    templatepath = portal_url.getRelativeContentURL(def_template)
    layouttool.setLayoutTemplateFor(portal, templatepath)
    out.write('Template set as portal default\n')
    out.write('Upgrading is now complete!')
    return out.getvalue()

def doDeleteOldLayout(self):
    """
    You must have saved some data before installing the new version
    of CMFPortalLayout on your filesystem.
    We will use the backup file created from the 'beforeUpgrade' method
    and install the new version of CMFPortalLayout in your Zope system.
    """
    out.write('Deleting your old Portal Layout installation\n\n')
    portal = getToolByName(self, 'portal_url').getPortalObject()
    deleteOldLayout(portal)
    
def deleteOldLayout(checkFolder):
    """
    Delete old layout settings
    """
    out.write('Deleting old layout from %s\n' % checkFolder.absolute_url(relative=1))
    for obj in checkFolder.objectValues():
        if obj.getId()[:3] == 'clf' and obj.meta_type == 'Folder':
            objid = obj.getId()
            checkFolder.manage_delObjects(objid)
            out.write('Deleted %s\n' % objid)
        else:
            try:
                delids = []
                if obj.hasProperty('localColumnList'):
                    delids.append('localColumnList')
                if obj.hasProperty('columnList'):
                    delids.append('columnList')
                if obj.hasProperty('mainColumn'):
                    delids.append('mainColumn')
                if delids:
                    obj.manage_delProperties(delids)
                delids = None
            except:
                out.write('Deleting properties for %s failed' % obj.absolute_url(relative=1))
        if obj.isPrincipiaFolderish:
            deleteOldLayout(obj)