##############################################################################
#    Copyright (C) 2003  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################


import string
import zLOG
from OFS.Folder import Folder
from AccessControl import ClassSecurityInfo

from Products.CMFCore.utils import getToolByName

import LayoutPermissions

class BaseColumn(Folder):

    sort_order = 0
    security = ClassSecurityInfo()

    def __cmp__(self, other):
        """
        Compare function - used when sorting columns
        """
        if other is not None:
            return cmp(self.sort_order, other.sort_order)
        else:
            return 1

    def manage_afterAdd(self, item, container):
        """
        Set the sort order of the layout column
        """
        sort_order = 0
        for e in container.objectValues(['Portal Layout Column', 'Portal Layout Main Column']):
            try:
                if e.getId() != item.getId() and e.sort_order >= sort_order:
                    sort_order = (e.sort_order + 1)
            except:
                zLOG.LOG(self.meta_type, zLOG.INFO, 'manage_afterAdd: exception %s - %s\n%s' % (sys.exc_type, sys.exc_value, traceback.format_list(traceback.extract_tb(sys.exc_traceback))))

        self.sort_order = sort_order

    security.declareProtected(LayoutPermissions.View, 'getSortOrder')
    def getSortOrder(self):
        """
        Returns the sort order for the column
        """
        if hasattr(self, 'sort_order'):
            return self.sort_order
        else:
            return None

    security.declareProtected(LayoutPermissions.ChangeLayoutElement, 'setSortOrder')
    def setSortOrder(self, sort_order=None):
        """
        Set the sort order for the column
        """
        self.sort_order = sort_order

    security.declareProtected(LayoutPermissions.ChangeLayoutColumn, 'edit')
    def edit(self, title=None, width=None, useBorder=None, REQUEST=None):
        """
        Edit the topbar settings
        """
        if title is not None:
            self.title = title

        if width is not None:
            self.width = width

        if useBorder is not None:
            if self.meta_type == 'Portal Layout Main Column':
                self.useBorder = int(useBorder)

        if REQUEST is not None:
            msg = 'Saved column settings'
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

class MainColumn(BaseColumn):
    """
    A Main Column is used to define main slot
    """

    meta_type = 'Portal Layout Main Column'
    sort_order = 0
    security = ClassSecurityInfo()
    useBorder = 0

    def __init__(self, id, title=''):
        """
        Construct new main column
        """
        self.id = id
        self.title = title
        self.width = '100%'


class LayoutColumn(BaseColumn):
    """
    A Layout Column is used to put layout elements in
    """

    meta_type = 'Portal Layout Column'
    sort_order = 0
    security = ClassSecurityInfo()

    def __init__(self, id, title=''):
        """
        Construct new layout column
        """
        self.id = id
        self.title = title
        self.width = '100%'

    security.declareProtected(LayoutPermissions.AddLayoutElement, 'addLayout')
    def addLayout(self, layout_type, layout_title=None, REQUEST=None):
        """
        Add a new layout element to the column
        """
        layout = None
        ltool = getToolByName(self, 'portal_layout')
        for lt in ltool.listLayoutClasses():
            if layout_type == lt.meta_type:
                layout = lt(layout_title)
                break

        if layout is None:
            # No layout type matched passed in value
            raise NameError, 'Unknown layout type: %s' % layout_type

        self._setObject(layout.getId(), layout)
        if REQUEST is not None:
            msg = 'Added new %s' % layout_type
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declareProtected(LayoutPermissions.DeleteLayoutElement, 'deleteLayout')
    def deleteLayout(self, layoutid, REQUEST=None):
        """
        Delete the requested layout
        """
        if hasattr(self, layoutid):
            layout = getattr(self, layoutid)
            layouttitle = layout.title_or_id()
            self.manage_delObjects(layoutid)
            msg= 'Deleted layout %s' % layouttitle
        else:
            msg = 'Layout with id %s not found' % layoutid

        if REQUEST is not None:
            req_url = REQUEST.get('HTTP_REFERER')
            if not req_url:
                req_url = REQUEST.get('returnToUrl')
                if not req_url:
                    req_url = "%s/column_edit_form" % self.absolute_url()
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            zLOG.LOG("%s - del layout" % self.meta_type, zLOG.INFO, "Req url is %s" % req_url)
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declareProtected(LayoutPermissions.ChangeLayoutElement, 'moveLayout')
    def moveLayout(self, layout_id, direction, REQUEST=None):
        """
        Move selected layout
        """
        swapobj = None
        swap1 = getattr(self, layout_id)
        objs = self.getLayoutElements()

        if direction == 'up':
            for layout in objs:
                if swapobj is None: swapobj = layout
                if layout.getId() != id:
                    if layout.sort_order < swap1.sort_order and layout.sort_order > swapobj.sort_order:
                        swapobj = layout
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)
        elif direction == 'down':
            for layout in objs:
                if swapobj is None: swapobj = layout
                if layout.getId() != id:
                    if layout.sort_order > swap1.sort_order and layout.sort_order > swapobj.sort_order:
                        swapobj = layout
                        break
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)

        if REQUEST is not None:
            msg = 'Moved layout %s' % direction
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declareProtected(LayoutPermissions.View, 'getLayoutElements')
    def getLayoutElements(self):
        """
        Return a sorted list of layout elements
        """
        layouts = []
        if self.objectValues():
            ltool = getToolByName(self, 'portal_layout')
            layouts = list(self.objectValues(ltool.listLayoutTypes(filter=0)))
            layouts.sort()
        return layouts

    security.declareProtected(LayoutPermissions.View, 'getLayoutElementsForLanguage')
    def getLayoutElementsForLanguage(self):
        """
        Returns a sorted list of layout elements
        Uses language to determine what elements should be returned
        """
        layouts = self.getLayoutElements()
        ret_layouts = []
        ltool = getToolByName(self, 'portal_layout')
        try:
            msgcatalog = ltool.restrictedTraverse(ltool.getMessageCatalog())
            sel_language = msgcatalog.get_selected_language()
        except:
            sel_language = 'en'
        for layout in layouts:
            lan = layout.getLanguage()
            if lan and lan != sel_language:
                continue
            else:
                ret_layouts.append(layout)

        return ret_layouts
