##############################################################################
#    Copyright (C) 2003  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import string
import zLOG
from OFS.Folder import Folder
from AccessControl import ClassSecurityInfo

from Products.CMFCore.utils import getToolByName

from utils import getUniqueId
import LayoutPermissions
from LayoutColumn import LayoutColumn, MainColumn

class LayoutRow(Folder):
    """
    A Layout Row defines a row containing Layout Columns
    """

    meta_type = 'Portal Layout Row'
    sort_order = 0
    security = ClassSecurityInfo()
    useBorder = 0
    cellpadding = 0
    cellspacing = 0

    def __init__(self, id, title=''):
        """
        Construct new row
        """
        self.id = id
        self.title = title
        self.useBorder = 0
        self.cellpadding = 0
        self.cellspacing = 0

    def __cmp__(self, other):
        """
        Compare function - used when sorting rows
        """
        if other is not None:
            return cmp(self.sort_order, other.sort_order)
        else:
            return 1

    def manage_afterAdd(self, item, container):
        """
        Set the sort order of the layout row
        """
        zLOG.LOG("%s - afterAff" % self.meta_type, zLOG.INFO, 'Analyzing items:\nself: %s\nitem: %s\ncontainer: %s' % \
            (str(self), str(item), str(container)))
        ptemplate = None
        if item.meta_type == 'Portal Layout Template':
            ptemplate = item
        else:
            ptemplate = container
        sort_order = 0
        for e in ptemplate.objectValues('Portal Layout Row'):
            zLOG.LOG(self.meta_type, zLOG.INFO, 'E is %s and has sorder %d' % \
                (e.absolute_url(), e.sort_order))
            try:
                if e.getId() != self.getId() and e.sort_order >= sort_order:
                    sort_order = (e.sort_order + 1)
            except:
                zLOG.LOG(self.meta_type, zLOG.INFO, 'manage_afterAdd: exception %s - %s\n%s' % (sys.exc_type, sys.exc_value, traceback.format_list(traceback.extract_tb(sys.exc_traceback))))

        self.sort_order = sort_order

        #Create new layout element
        ltool = getToolByName(item, 'portal_layout')
        for lt in ltool.listLayoutClasses():
            if lt.meta_type == 'Layout Element':
                layout_element = lt()
                break
        elemid = layout_element.id
        self._setObject(elemid, layout_element)
        zLOG.LOG(self.meta_type, zLOG.INFO, 'Container is %s and has type %s' % \
            (container.absolute_url(), container.meta_type))
        if ptemplate.getMainRowId() == self.id:
            layout_element = getattr(self, elemid)
            colid = getUniqueId()
            maincol = MainColumn(colid, 'Main')
            layout_element._setObject(colid, maincol)


    security.declareProtected(LayoutPermissions.View, 'getSortOrder')
    def getSortOrder(self):
        """
        Returns the sort order for the row
        """
        if hasattr(self, 'sort_order'):
            return self.sort_order
        else:
            return None

    security.declareProtected(LayoutPermissions.ChangeLayoutElement, 'setSortOrder')
    def setSortOrder(self, sort_order=None):
        """
        Set the sort order for the row
        """
        self.sort_order = sort_order

    security.declareProtected(LayoutPermissions.ChangeLayoutColumn, 'edit')
    def edit(self, title=None, useBorder=None, cellpadding=None, cellspacing=None, REQUEST=None):
        """
        Edit the layout row settings
        """
        if title is not None:
            self.title = title

        if useBorder is not None:
            self.useBorder = int(useBorder)
            
        if cellpadding is not None:
            self.cellpadding = int(cellpadding)

        if cellspacing is not None:
            self.cellspacing = int(cellspacing)

        if REQUEST is not None:
            msg = 'Saved layout row'
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declareProtected(LayoutPermissions.View, 'getLayoutElement')
    def getLayoutElement(self):
        """
        Return the layout element of the row
        """
        layouts = self.objectValues('Layout Element')
        if layouts:
            return layouts[0]
        else:
            raise AttributeError, 'No layout element added!!'