##############################################################################
#    Copyright (C) 2003  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import zLOG
import sys, traceback
from Globals import InitializeClass
from OFS.Folder import Folder
from OFS.SimpleItem import SimpleItem
from Acquisition import Implicit, aq_parent, aq_inner, aq_base
from AccessControl import ClassSecurityInfo
from Persistence import Persistent
from ExtensionClass import Base
from Products.PageTemplates import PageTemplate
from Products.PageTemplates.ZopePageTemplate import SecureModuleImporter

from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
from Products.CMFCore.DynamicType import DynamicType

from Products.CMFDefault.utils import *

from utils import getUniqueId
from LayoutRow import LayoutRow
import LayoutPermissions

factory_type_information = ({'id': 'Portal Template'
                           , 'meta_type': 'Portal Layout Template'
                           , 'description': 'Use templates to create layout for pages'
                           , 'icon': 'layouttemplate_icon.gif'
                           , 'product': 'CMFPortalLayout'
                           , 'factory': 'addPortalLayoutTemplate'
                           , 'filter_content_types': 0
                           , 'immediate_view': 'template_edit_form'
                           , 'actions': ({'id': 'view'
                                        , 'name': 'View'
                                        , 'action': 'index_html'
                                        , 'permissions': (LayoutPermissions.View,)
                                        , 'category': 'object'}
                                       , {'id': 'edit'
                                        , 'name': 'Edit'
                                        , 'action': 'template_edit_form'
                                        , 'permissions': (LayoutPermissions.ChangeLayoutTemplate,)
                                        , 'category': 'object'})
                           },)

class LayoutPageTemplate(PageTemplate.PageTemplate, Implicit, Persistent):
    """
    The PageTemplate of the Layout Template
    """

    def pt_getContext(self):
        root = self.getPhysicalRoot()
        template = self #aq_parent(self)
        here = aq_parent(template)
        c = {'template': template,
              'options': {},
             'nothing': None,
             'root': root,
             'request': getattr(root, 'REQUEST', None),
             'modules': SecureModuleImporter,
             'here': here,
             'container': aq_inner(here),
             }
        return c

    def __setstate__(self, state):
        LayoutPageTemplate.inheritedAttribute('__setstate__')(self, state)
        self._cook()

class Macros(Base):
    """
    Evaluate to the macros collection of the contained page template.
    """
    def __of__( self, doc):
        return doc.get_pt().macros


class LayoutTemplate(DynamicType, CMFCatalogAware, Folder):

    """
    A Layout Template is used to define a layout you can set on your portal and/or folders
    """

    meta_type = 'Portal Layout Template'

    manage_options = (Folder.manage_options[:3] + SimpleItem.manage_options)

    security = ClassSecurityInfo()

    #security.declareProtected(LayoutPermissions.View, 'macros')

    #_default_content_fn = os.path.join(package_home(globals()), 'www', 'mediaskin.html')

    def __init__(self, id, title='', description='', text=None):
        """
        Construct a new layout template
        """

        self._pt = LayoutPageTemplate()
        self.id = id
        self.title = title
        self.description = description
        self.pagewidth = '100%'
        self.align = 'center'
        self.cellpadding = 0
        self.cellspacing = 0
        self._main_row = None  #The id of the row that contains the main column
        """
        if text is None:
            text = open(self._default_content_fn).read()
        self._pt.write(text)
        """
        if text is not None:
            self._pt.write(text)

    def manage_afterAdd(self, item, container):
        """
        Add a new main column to a row in template
        """
        zLOG.LOG("%s - afterAff" % self.meta_type, zLOG.INFO, 'Analyzing items:\nself: %s\nitem: %s\ncontainer: %s' % \
            (str(self), str(item), str(container)))
        zLOG.LOG(self.meta_type, zLOG.INFO, 'selfpath: %s - itempath: %s' % \
            (self.absolute_url(relative=1), item.absolute_url(relative=1)))
        Folder.manage_afterAdd(self, item, container)
        zLOG.LOG(self.meta_type, zLOG.INFO, 'Main row is %s' % str(self._main_row))
        if not self._main_row:
            rowid = getUniqueId()
            colid = getUniqueId()
            item.setMainRow(rowid)
            zLOG.LOG(self.meta_type, zLOG.INFO, 'Creating new portal row :-(')
            mainrow = LayoutRow(rowid, 'Main')
            item._setObject(rowid, mainrow)
        """
        except:
            zLOG.LOG(self.meta_type, zLOG.INFO, 'Error in manage_afterAdd %s - %s\n%s' % \
                (sys.exc_info()[0]
               , sys.exc_info()[1]
               , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
        """
        if aq_base(container) is not aq_base(self):
            self.indexObject()
        
    def manage_beforeDelete(self, item, container):
        """
        Before delete, uncatalog
        """
        if aq_base(container) is not aq_base(self):
            self.unindexObject()
        
    security.declareProtected(LayoutPermissions.ChangeLayoutTemplate, 'setTitle')
    def setTitle(self, title=None):
        """
        Set the title
        """
        if title is not None:
                self.title = title
                
    security.declareProtected(LayoutPermissions.ChangeLayoutTemplate, 'setDescription')
    def setDescription(self, description=None):
        """
        Set the description
        """
        if description is not None:
            self.description = description
            
    security.declareProtected(LayoutPermissions.View, 'Title')
    def Title(self):
        """
        Return the title
        """
        return self.title
    
    security.declareProtected(LayoutPermissions.View, 'Description')
    def Description(self):
        """
        Return the description
        """
        return self.description


    security.declareProtected(LayoutPermissions.View, 'Type')
    def Type( self ):
        """
        Implement dublin core type
        """
        if hasattr(aq_base(self), 'getTypeInfo'):
            ti = self.getTypeInfo()
            if ti is not None:
                return ti.Title()
        return self.meta_type
    
    security.declareProtected(LayoutPermissions.ChangeLayoutTemplate, 'edit')
    def edit(self, title=None, description=None, text='', pagewidth=None, align=None, cellpadding=None, cellspacing=None, REQUEST=None):
        """
        Edit the layout template
        """

        if title is not None:
           self.title = title

        if description is not None:
            self.description = description

        if pagewidth is not None:
            self.pagewidth = pagewidth

        if align is not None:
            if align == 'None':
                align = None
            self.align = align

        if cellpadding is not None:
            self.cellpadding = int(cellpadding)

        if cellspacing is not None:
            self.cellspacing = int(cellspacing)

        """
        text = bodyfinder(text)
        self._edit(text=text)
        """
        self.reindexObject()

        if REQUEST is not None:
            msg = 'Saved template settings'
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            if REQUEST.has_key('frameable'):
                extra = '&frameable=true'
            else:
                extra = ''
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s%s' % (req_url, msg.replace(' ', '+'), extra))

    def SearchableText(self):
        """
           Used by the catalog for basic full text indexing.
        """
        return "%s %s %s" % (self.Title(), self.Description(), self.get_pt().read())

    security.declareProtected(LayoutPermissions.AddLayoutColumn, 'addLayoutRow')
    def addLayoutRow(self, REQUEST=None):
        """
        Add a new layout row
        """
        row = LayoutRow(getUniqueId())
        self._setObject(row.getId(), row)
        if REQUEST is not None:
            msg = 'Added new layout row'
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            if REQUEST.has_key('frameable'):
                extra = '&frameable=true'
            else:
                extra = ''
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s%s' % (req_url, msg.replace(' ', '+'), extra))

    security.declareProtected(LayoutPermissions.DeleteLayoutColumn, 'deleteLayoutRow')
    def deleteLayoutRow(self, rowid, REQUEST=None):
        """
        Delete the requested layout row
        """
        if hasattr(self, rowid):
            row = getattr(self, rowid)
            rowtitle = row.title_or_id()
            self.manage_delObjects(rowid)
            msg = 'Deleted row %s' % rowtitle
        else:
            msg = 'Row with id %s not found' % rowid

        if REQUEST is not None:
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            if REQUEST.has_key('frameable'):
                extra = '&frameable=true'
            else:
                extra = ''
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s%s' % (req_url, msg.replace(' ', '+'), extra))

    security.declareProtected(LayoutPermissions.MoveLayoutColumn, 'moveLayoutRow')
    def moveLayoutRow(self, rowid, direction, REQUEST=None):
        """
        Move the requested layout row in direction
        """
        swapobj = None
        swap1 = getattr(self, rowid)
        rows = self.getLayoutRows()

        if direction == 'up':
            for row in rows:
                if swapobj is None: swapobj = row
                if row.getId() != id:
                    if row.sort_order < swap1.sort_order and row.sort_order > swapobj.sort_order:
                        swapobj = row
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)
        elif direction == 'down':
            for row in rows:
                if swapobj is None: swapobj = row
                if row.getId() != id:
                    if row.sort_order > swap1.sort_order and row.sort_order > swapobj.sort_order:
                        swapobj = row
                        break
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)

        if REQUEST is not None:
            msg = 'Moved row %s' % direction
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            if REQUEST.has_key('frameable'):
                extra = '&frameable=true'
            else:
                extra = ''
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s%s' % (req_url, msg.replace(' ', '+'), extra))

    security.declareProtected(LayoutPermissions.View, 'getLayoutRows')
    def getLayoutRows(self):
        """
        Get the layout rows sorted for this template
        """
        rows = []
        if self.objectValues('Portal Layout Row'):
            rows = list(self.objectValues('Portal Layout Row'))
            rows.sort()
        return rows

    security.declareProtected(LayoutPermissions.ChangeLayoutTemplate, 'setMainRow')
    def setMainRow(self, rowid, REQUEST=None):
        """
        Set the id of the row that contains the main column
        """
        row = getattr(self, rowid, None)
        if row:
            self._main_row = rowid
            msg = 'Main row set to %s' % row.title_or_id()
        else:
            msg = 'Row with id %s not found' % rowid

        if REQUEST is not None:
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declarePublic('getMainRow')
    def getMainRow(self):
        """
        Return the main row
        If not found - use the first found row (and in this row, use the first column as main)
        Missing main row and column should *never* happen
        """
        main = None
        if self._main_row:
            main = getattr(self, self._main_row, None)
        if not main:
            rows = self.getLayoutRows()
            for row in rows:
                layoutelem = row.getLayoutElement()
                if layoutelem.objectValues('Portal Layout Main Column'):
                    main = row
                    break
            if rows and not main:
                main = rows[0]
        return main

    security.declarePublic('getMainRowId')
    def getMainRowId(self):
        """
        Return the id of the main column
        """
        mainrow = self.getMainRow()
        rowid = None
        if mainrow:
            rowid = mainrow.getId()
        return rowid

    #PageTemplate stuff below !!
    security.declareProtected(LayoutPermissions.ChangeLayoutTemplate, 'reset')
    def reset(self):
        """
        Reset the text to the default.
        """
        text = open(self._default_content_fn).read()
        self._pt.write(text)

    def get_pt(self):
        """
            return wrapped PageTemplate object.
        """
        return self._pt.__of__(self)

    def _edit(self, text):
        """
          Edit the document
        """
        pt = self.get_pt()
        pt.write(text)

    def CookedBody(self):
        """
           Return a rendering of the Dynamic Document.
        """
        security = getSecurityManager()
        security.addContext(self)
        try:
            try:
                result = self.get_pt().pt_render(
                    extra_context={'user': security.getUser()})
            except TALESError, err:
                if err.type == 'Unauthorized':
                    raise err.type, err.value, err.traceback
                raise
            return result
        finally:
            security.removeContext(self)

    def EditableBody(self):
        """
          The editable body of text.  This is the raw structured text
          or HTML.
        """
        return self.get_pt().read()

    def Format(self):
        """
            Returns a content-type style format of the underlying source
        """
        return self.get_pt().content_type

    def setFormat(self, value):
        """
           Ignore the value, as we only support text/html currently
        """
        self.get_pt().content_type = 'text/html'

InitializeClass(LayoutTemplate)

def addPortalLayoutTemplate(self, id, title=None, REQUEST=None):
    """
    Add a new Portal Layout Template
    """
    ob = LayoutTemplate(id, title)
    self._setObject(id, ob)
