##############################################################################
#    Copyright (C) 2002-03  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################


import os, zLOG, random, sys, re, string, traceback
from Globals import InitializeClass, package_home
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_parent, aq_inner
from OFS.Folder import Folder
from OFS.SimpleItem import SimpleItem
from Products.PageTemplates import PageTemplateFile
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.Expression import Expression
from DateTime import DateTime
from FormatObject import FormatObject
from utils import getUniqueId
from LayoutTemplate import LayoutTemplate
from LayoutColumn import LayoutColumn
import LayoutPermissions

_zptdir = os.path.join(package_home(globals()), 'www')

webcolors = ('aqua','black','blue','fuchsia','gray','green','lime','maroon','navy','olive','purple','red','silver','teal','white','yellow')

class LayoutTool(UniqueObject, Folder, ActionProviderBase):

    """
    A tool for creating portal layouts
    """

    id = 'portal_layout'
    meta_type = 'Layout Tool'
    colors = webcolors
    _layout_types = []
    _available_types = []
    _page_layouts = {}
    _templates_for_types = {}
    _msgcatalog = 'cmf_portallayout'
    _maincolumn = None
    COLUMN_PREFIX = 'clf_'
    TOPBAR_PREFIX = 'clftop_'

    _format_elements = [ # default format element list - mostly Dublin Core
                 {"title":"Title","expression":"obj/title_or_id"}
                ,{"title":"Description","expression":"obj/Description"}
                ,{"title":"Subject","expression":"python:modules['string'].join(obj.Subject(),', ')"}
                ,{"title":"Creator","expression":"obj/Creator"}
                ,{"title":"Publisher","expression":"obj/Publisher"}
                ,{"title":"Contributors","expression":"python:strong.join(obj.Contributors(),', ')"}
                ,{"title":"Type","expression":"obj/Type"}
                ,{"title":"CreationDate","expression":"obj/CreationDate"}
                ,{"title":"ModificationDate","expression":"obj/ModificationDate"}
                ,{"title":"EffectiveDate","expression":"obj/EffectiveDate"}
                ,{"title":"ExpirationDate","expression":"obj/ExpirationDate"}
                ,{"title":"Format","expression":"obj/Format"}
                ,{"title":"Language","expression":"obj/Language"}
                ,{"title":"Rights","expression":"obj/Rights"}
                ,{"title":"Icon","expression":"python:obj.restrictedTraverse(obj.getIcon())"}
                ,{"title":"Status", "expression":"python:portal.portal_workflow.getInfoFor(obj, 'review_state', '')"}
                ,{"title":"PrintIcon", "expression":'string: <img src="print.gif" alt="Print" />'}
                ,{"title":"EventStart","expression":"python:hasattr(obj, 'start') and obj.start().strftime('%d/%m-%Y %H:%M') or ''"}
                ,{"title":"EventEnd", "expression":"python: hasattr(obj, 'end') and obj.end(),strftime('%d/%m-%Y %H:%M') or ''"}
                #,NEWLINE:{"expression":""}
               ]

    _styles = [
               {"title":"Heading","font_size":"","font_weight":"bold","font_style":"","font_color":"", "link_image":"", "image_position":"", "image_padding":""}
              ,{"title":"LargeHeading","font_size":"120%","font_weight":"bold","font_style":"","font_color":"", "link_image":"", "image_position":"", "image_padding":""}
              ,{"title":"Description","font_size":"","font_weight":"","font_style":"italic","font_color":"", "link_image":"", "image_position":"", "image_padding":""}
              ]

    _links = [{'title': 'CreatorHome', 'expression': 'python:portal.portal_membership.getHomeUrl(obj.Creator())'}
                , {'title': 'ObjectHome', 'expression': 'obj/absolute_url'}
                , {'title': 'ObjectPrint', 'expression': 'string:${obj/absolute_url}?printable=true'}
                 ]

    _actions = [ ActionInformation(
                    id='edit_layout'
                  , title='Layout'
                  , action=Expression(
                        text="string: javascript:void(window.open('${folder_url}/setupLayout', 'edit_layout', 'location=no, menubar=no, personalbar=no, status=yes, resizable=yes, scrollbars=yes'));")
                  , condition=''
                  , permissions=(CMFCorePermissions.ManageProperties,)
                  , category='folder'
                  , visible=1
                  )
                , ActionInformation(
                    id='edit_layout_templates'
                  , title='Portal Templates'
                  , action=Expression(
                        text="string: javascript:void(window.open('${folder_url}/layouttemplates_edit_form', 'edit_templates', 'location=no, menubar=no, personalbar=no, status=yes, resizable=yes, scrollbars=yes'));")
                  , condition=Expression(text="python:'portal_configuration' not in portal.objectIds()")
                  , permissions=(CMFCorePermissions.ManagePortal,)
                  , category='global'
                  , visible=1
                  )
                , ActionInformation(
                    id='edit_layout_templates'
                  , title='Portal Templates'
                  , action=Expression(
                        text="string: ${folder_url}/layouttemplates_edit_form")
                  , condition=Expression(text="python:'portal_configuration' in portal.objectIds()")
                  , permissions=(CMFCorePermissions.ManagePortal,)
                  , category='configuration'
                  , visible=1
                  )
                ,ActionInformation(
                    id='layout_types_edit'
                  , title='Types'
                  , action=Expression(
                        text='string: ${portal_url}/layouttypes_editform')
                  , condition=None
                  , permissions=(CMFCorePermissions.ManagePortal,)
                  , category='configuration'
                  , visible=1
                  )
                , ActionInformation(
                    id='layout_types_templates'
                  , title='Template Types'
                  , action=Expression(
                      text='string: ${portal/portal_layout/absolute_url}/layouttypetemplates_editform')
                  , condition=None
                  , permissions=(CMFCorePermissions.ManagePortal,)
                  , category='configuration'
                  , visible=1
                  )
                , ActionInformation(
                    id='layout_styles_edit'
                  , title='Styles'
                  , action=Expression(
                        text='string: ${portal_url}/layoutstyles_editform')
                  , condition=None
                  , permissions=(CMFCorePermissions.ManagePortal,)
                  , category='configuration'
                  , visible=1
                  )
                , ActionInformation(
                    id='layout_formats_edit'
                  , title='Formats'
                  , action=Expression(
                        text='string: ${portal_url}/layoutformats_editform')
                  , condition=None
                  , permissions=(CMFCorePermissions.ManagePortal,)
                  , category='configuration'
                  , visible=1
                  )
                , ActionInformation(
                   id='msgcatalog_edit'
                 , title='Message Catalog'
                 , action=Expression(text='string:${portal_url}/edit_layout_msgcatalog')
                 , condition=None
                 , permissions=(CMFCorePermissions.ManagePortal,)
                 , category='configuration'
                 , visible=1
                 )
                #, ActionInformation(
                #  id='sitestyles_edit'
                #, title='Site Styles'
                #, action=Expression(text='string:${portal_url}/sitestyles_edit_form')
                #, condition=None
                #, permissions=(CMFCorePermissions.ManagePortal,)
                #, category='configuration'
                #, visible=1
                #)
               ]

    security = ClassSecurityInfo()

    manage_options = ( { 'label' : 'Overview', 'action' : 'manage_overview' }
                      , Folder.manage_options[0]
                     , { 'label' : 'Types', 'action' : 'manage_Types' }
                     , { 'label' : 'Type Templates', 'action' : 'manage_TypeTemplates' }
                     , { 'label' : 'Formats', 'action' : 'manage_Formats' }
                     , { 'label' : 'Format Elements', 'action' : 'manage_FormatElements' }
                     , { 'label' : 'Styles', 'action' : 'manage_Styles' }
                     , { 'label' : 'Links', 'action' : 'manage_Links' }
                     , { 'label' : 'Roles', 'action' : 'manage_setRoleTypesForm' }
                     , { 'label' : 'Groups', 'action' : 'manage_setGroupTypesForm' }
                     , { 'label' : 'Users', 'action' : 'manage_setUserTypesForm' }
                     ) + ActionProviderBase.manage_options + SimpleItem.manage_options

    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_overview' )
    manage_overview = PageTemplateFile.PageTemplateFile( 'explainLayoutTool', _zptdir)

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_Types' )
    manage_Types = PageTemplateFile.PageTemplateFile( 'layouttoolSetTypes', _zptdir)

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_Formats' )
    manage_Formats = PageTemplateFile.PageTemplateFile( 'layouttoolFormats', _zptdir)

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_renameFormatsForm' )
    manage_renameFormatsForm = PageTemplateFile.PageTemplateFile( 'layouttoolRenameFormat_form', _zptdir)

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_FormatElements' )
    manage_FormatElements = PageTemplateFile.PageTemplateFile( 'formatElementLines', _zptdir)

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_Styles' )
    manage_Styles = PageTemplateFile.PageTemplateFile( 'formatStyles', _zptdir)

    security.declareProtected( CMFCorePermissions.ManagePortal
                            , 'manage_Links' )
    manage_Links = PageTemplateFile.PageTemplateFile( 'formatLinks', _zptdir)
    
    security.declareProtected(CMFCorePermissions.ManagePortal
                            , 'manage_TypeTemplates')
    manage_TypeTemplates = PageTemplateFile.PageTemplateFile('layouttoolTypeTemplates', _zptdir)

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setRoleTypesForm')
    manage_setRoleTypesForm = PageTemplateFile.PageTemplateFile('layouttoolSetRoleTypes', _zptdir )

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setGroupTypesForm')
    manage_setGroupTypesForm = PageTemplateFile.PageTemplateFile('layouttoolSetGroupTypes', _zptdir)

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setUserTypesForm')
    manage_setUserTypesForm = PageTemplateFile.PageTemplateFile('layouttoolSetUserTypes', _zptdir)


    #
    #   'portal_menutool' interface methods
    #

    #GENERAL METHODS

    security.declareProtected(CMFCorePermissions.View, 'all_meta_types')
    def all_meta_types(self):
        """ satisfy copy support """
        return [{"name":"Format Object","action":"manage_addFormat"},]

    security.declarePublic('findAllLayoutObjects')
    def findAllLayoutObjects(self):
        """
        Return a list of all availiable layout objects
        """
        portal = getToolByName(self, 'portal_url').getPortalObject()
        findres = self.PrincipiaFind(portal, obj_metatypes=self.listLayoutTypes(), search_sub=1)
        res = []
        for findtuple in findres:
            parent = findtuple[0]
            obj = findtuple[1]
            res.append(obj)
        return res


    security.declarePublic('getSortByIndexes')
    def getSortByIndexes(self):
        """
        Return the ids of indexes which can be used as sort_on index
        """
        sort_on = []
        catalog = self.portal_catalog
        indexes = catalog._catalog.indexes
        for index in indexes.keys():
            if hasattr(indexes[index], 'keyForDocument'):
                sort_on.append(index)
        sort_on.sort()
        return sort_on

    security.declarePrivate('listActions')
    def listActions(self, info=None):
        """
        Return actions provided by tool
        """
        return self._actions

    security.declarePublic('getId')
    def getId(self):
        """
        Return the id
        """
        return self.id

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setGlobalRolesTypes')
    def manage_setGlobalRolesTypes(self, REQUEST=None):
        """
        Change which types a specific role can add
        """
        if REQUEST is not None:
            allowed_roles_for_types = {}
            current_types = REQUEST['current_types']
            i = 0
            for type in current_types:
                allowed_roles_for_types[type] = REQUEST.get('type_%d' % i, [])
                i = i + 1
            self._allowed_roles_for_types = allowed_roles_for_types
            self._p_changed = 1
            if not REQUEST.has_key('no_redirect'):
                return self.manage_setRoleTypesForm(
                    self, REQUEST, manage_tabs_message='Properties changed.')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setGlobalGroupTypes')
    def manage_setGlobalGroupTypes(self, REQUEST):
        """
        Change which types a specific group can add
        """
        allowed_groups_for_types = {}
        current_types = REQUEST['current_types']
        i = 0
        for type in current_types:
            allowed_groups_for_types[type] = REQUEST.get('type_%d' % i, [])
            i = i + 1
        self._allowed_groups_for_types = allowed_groups_for_types
        self._p_changed = 1
        if not REQUEST.has_key('no_redirect'):
            return self.manage_setGroupTypesForm(self, REQUEST, manage_tabs_message='Groups changed')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setGlobalUserTypes')
    def manage_setGlobalUserTypes(self, REQUEST):
        """
        Change which types a specific user can add
        """
        allowed_users_for_types = {}
        current_types = REQUEST['current_types']
        i = 0
        for type in current_types:
            allowed_users_for_types[type] = REQUEST.get('type_%d' % i, [])
            i = i + 1
        self._allowed_users_for_types = allowed_users_for_types
        self._p_changed = 1
        if not REQUEST.has_key('noredirect'):
            return self.manage_setUserTypesForm(self, REQUEST, manage_tabs_message='Users changed')

    security.declarePrivate('addUserTypes')
    def addUserTypes(self, username, types):
        """
        Change the user settings
        """
        zLOG.LOG(self.meta_type, zLOG.INFO, 'Add user types called. Username: %s, Types: %s' \
        % (username, str(types)))
        if not hasattr(self, '_allowed_users_for_types'):
            self._allowed_users_for_types = {}

        allowed_user_types = self._allowed_users_for_types
        all_types = self.listLayoutTypes()

        for typeid in all_types:
            if typeid in types:
                users = []
                if allowed_user_types.has_key(typeid):
                    users = allowed_user_types[typeid]
                if username not in users:
                    users.append(username)
                    allowed_user_types[typeid] = users

        #zLOG.LOG(self.meta_type, zLOG.INFO, 'Types after set %s' % str(allowed_user_types))
        self._allowed_users_for_types = allowed_user_types
        self._p_changed = 1

    def _getRoles(self, column):
        portal_membership = getToolByName(self, 'portal_membership')
        member = portal_membership.getAuthenticatedMember()
        if not portal_membership.isAnonymousUser():
            localroles = column.get_local_roles_for_userid(member.getUserName())
        roles = member.getRoles()
        if not portal_membership.isAnonymousUser():
            roles = roles + ('Member',)
        return roles

    security.declarePublic('listFilteredLayouts')
    def listFilteredLayouts(self, column):
        """
            List types that can be added to a layout column
        """
        portal_types = getToolByName(self, 'portal_types')
        membership = getToolByName(self, 'portal_membership')
        roles = self._getRoles(column)
        username = membership.getAuthenticatedMember().getUserName()
        try:
            groupnames = membership.getMemberGroups()
        except:
            groupnames = None
        result = []

        for ltype in self.listLayoutTypes():
            isAllowed = self.allowLayoutType(ltype, role=roles, username=username, groupname=groupnames)
            if isAllowed:
                result.append(ltype)
        return result

    security.declarePublic('allowLayoutType')
    def allowLayoutType(self, ltype, role=None, username=None, groupname=None):
        """
            Return true if layout can be added
        """
        if not hasattr(self, '_allowed_roles_for_types'):
            self._allowed_roles_for_types = {}
        if not hasattr(self, '_allowed_groups_for_types'):
            self._allowed_groups_for_types = {}
        if not hasattr(self, '_allowed_users_for_types'):
            self._allowed_users_for_types = {}

        roleallowed = 0
        userallowed = 0
        groupallowed = 0
        if role is not None:
            if self._allowed_roles_for_types.has_key(ltype):
                if type(role) is type([]) or type(role) is type(()):
                    for r in role:
                        if r in self._allowed_roles_for_types[ltype]:
                            roleallowed = 1
                else:
                    roleallowed = role in self._allowed_roles_for_types[ltype]

        if  username is not None:
            if self._allowed_users_for_types.has_key(ltype):
                userallowed = username in self._allowed_users_for_types[ltype]

        if groupname is not None:
            if self._allowed_groups_for_types.has_key(ltype):
                if type(groupname) is type([]) or type(groupname) is type(()):
                    for g in groupname:
                        if g in self._allowed_groups_for_types[ltype]:
                            groupallowed = 1
                else:
                    groupallowed = groupname in self._allowed_groups_for_types[ltype]
        return (roleallowed or userallowed or groupallowed)

    security.declarePublic('listLayoutTypes')
    def listLayoutTypes(self, filter=1):
        """
        Returns a list of meta_types
        """
        types = []
        for type in self._layout_types:
            if not type.meta_type == 'Ghost Element' or not filter:
                types.append(type.meta_type)
        return types

    security.declarePublic('listLayoutClasses')
    def listLayoutClasses(self):
        """
        Returns a list of available layout classes
        """
        return self._layout_types

    security.declarePublic('listAvailableTypes')
    def listAvailableTypes(self):
        """
        Returns a list of portal types available for list menus
        """
        avail_types = []
        types = self.portal_types.listContentTypes()
        for ltype in self._available_types:
            if ltype in types:
                avail_types.append(ltype)
        return avail_types
    
    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setTypeTemplates')
    def manage_setTypeTemplates(self, REQUEST):
        """
        Change which types have a specific template
        Overrides the default template setting for the portal
        """
        typetemplates = {}
        typestool = getToolByName(self, 'portal_types')
        types = typestool.listTypeInfo()
        for type in types:
            typename = type.Title()
            templatepath = REQUEST.get("%s_template" % typename, None)
            if templatepath == 'default':
                templatepath = None
            typetemplates[typename] = templatepath
        self._templates_for_types = typetemplates
        self._p_changed = 1
        if not REQUEST.has_key('noredirect'):
            return self.manage_TypeTemplates(self, REQUEST
                        , manage_tabs_message='Type templates changed')
        
    security.declarePublic('getTemplateForType')
    def getTemplateForType(self, typename):
        """
        Return the path of the template or None
        """
        templatepath = None
        if self._templates_for_types.has_key(typename):
            templatepath = self._templates_for_types[typename]
        return templatepath

    security.declarePublic('getMessageCatalog')
    def getMessageCatalog(self):
        """
        Return the name of the message catalog
        """
        return self._msgcatalog

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setMessageCatalog')
    def manage_setMessageCatalog(self, REQUEST):
        """
        Set the name of the message catalog
        """
        req = REQUEST
        msgcatalog = req.get('msgcatalog', None)
        if msgcatalog is not None:
            self._msgcatalog = msgcatalog
            msg = 'Message Catalog updated'
        else:
            msg = 'No message catalog selected'

        req_url = req.get('HTTP_REFERER', None)
        if not req_url:
            portal = getToolByName(self, 'portal_url').getPortalObject()
            req_url = '%s/edit_layout_messagecatalog' % portal.absolute_url()
        has_q = req_url.find('?')
        if has_q != -1:
            req_url,trash = req_url.split('?')
        req.RESPONSE.redirect('%s?frameable=true&portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setTypes')
    def manage_setTypes(self, current_types, noredirect=0):
        """
        Change the types available for list menus
        """
        self._available_types = current_types
        if not noredirect:
            return self.manage_Types(
                self, self.REQUEST, manage_tabs_message='Properties changed.')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_addFormat')
    def manage_addFormat(self, id='', title='', REQUEST=None):
        """
        Change the types available for list menus
        """
        format = FormatObject(id, title)
        self._setObject(format.getId(), format)

        if REQUEST is not None:
            return self.manage_Formats(
                self, self.REQUEST, manage_tabs_message='Format added.')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_renameFormat')
    def manage_renameFormat(self, REQUEST):
        """
        Rename a list of formats
        """
        self.manage_renameObjects(REQUEST['ids'], REQUEST['new_ids'], REQUEST)
        return self.manage_Formats(
            self, self.REQUEST, manage_tabs_message='Format renamed.')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_deleteFormat')
    def manage_deleteFormat(self, ids=(), REQUEST=None):
        """
        Delete select format object(s)
        """
        if type(ids) == type(()) or type(ids) == type([]):
            for id in ids:
                self._delObject(id)
        else:
            self._delObject(ids)

        if REQUEST is not None:
            return self.manage_Formats(self, self.REQUEST, manage_tabs_message='Format(s) deleted.')

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_editStyles' )
    def manage_editStyles( self, properties=None, REQUEST=None ):

        """ Update our list of styles.
        """
        if properties is None:
            properties = REQUEST

        styles = []

        for index in range( len( self._styles ) ):
            styles.append( self._extractStyle( properties, index ) )

        self._styles = tuple( styles )

        if REQUEST is not None:
            return self.manage_Styles(
                self, self.REQUEST, manage_tabs_message='Saved.')

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_addStyle')
    def manage_addStyle(self,title,  REQUEST=None):
        """
        Add a new style dictionary.
        """
        rgx = re.compile(r'(^_|[^a-zA-Z0-9-_~\,\.])')
        title = re.sub(rgx, "",title)
        self._testStyleTitleValidity(title)

        style = {
                "title":title
               ,"font_size":''
               ,"font_style":''
               ,"font_weight":''
               ,"font_color":''
               ,"link_image":''
               ,"image_position":''
               ,"image_padding":''
               }
        styles = list(self._styles)
        styles.append(style)
        self._styles = tuple(styles)
        self._p_changed = 1

        if REQUEST is not None:
            return self.manage_Styles(
                self, self.REQUEST, manage_tabs_message='%s added.' % title)

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_deleteStyles')
    def manage_deleteStyles(self, ids=(), REQUEST=None):
        """
        Delete the requested formatting
        """
        ids = list( map( int, ids ) )  # Convert to a list of integers.
        if ids:
            new_styles = []
            styles = list(self._styles)
            for idx in range(len(styles)):
                if idx not in ids:
                    new_styles.append(styles[idx])
            self._styles = tuple(new_styles)
            self._p_changed = 1
        if REQUEST:
            return self.manage_Styles(
                self, self.REQUEST, manage_tabs_message='Style(s) deleted.')

    # format elements
    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_editFormatElements' )
    def manage_editFormatElements( self, properties=None, REQUEST=None ):

        """ Update our list of format elements.
        """
        if properties is None:
            properties = REQUEST

        format_elements = []

        for index in range( len( self._format_elements ) ):
            format_elements.append( self._extractFormatElement( properties, index ) )

        self._format_elements = tuple( format_elements )

        if REQUEST is not None:
            return self.manage_FormatElements(
                self, self.REQUEST, manage_tabs_message='Saved.')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_addFormatElement')
    def manage_addFormatElement(self,title, expression, REQUEST=None):
        """
        Add a new format element dictionary.
        """
        format_element = {
                "title":title
               ,"expression":expression
               }
        format_elements = list(self._format_elements)
        format_elements.append(format_element)
        self._format_elements = tuple(format_elements)
        self._p_changed = 1

        if REQUEST is not None:
            return self.manage_FormatElements(
                self, self.REQUEST, manage_tabs_message='%s added.' % title)

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_deleteFormatElements')
    def manage_deleteFormatElements(self, ids=(), REQUEST=None):
        """
        Delete the requested format element
        """
        ids = list( map( int, ids ) )  # Convert to a list of integers.
        if ids:
            new_format_elements = []
            format_elements = list(self._format_elements)
            for idx in range(len(format_elements)):
                if idx not in ids:
                    new_format_elements.append(format_elements[idx])
            self._format_elements = tuple(new_format_elements)
            self._p_changed = 1
        if REQUEST:
            return self.manage_FormatElements(
                self, self.REQUEST, manage_tabs_message='Format element(s) deleted.')

    # link elements
    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_editLinks' )
    def manage_editLinks( self, properties=None, REQUEST=None ):

        """ Update our list of format elements.
        """
        if properties is None:
            properties = REQUEST

        link_elements = []

        for index in range( len( self._links) ):
            link_elements.append( self._extractLinkElement( properties, index ) )

        self._links = tuple( link_elements )

        if REQUEST is not None:
            return self.manage_Links(
                self, self.REQUEST, manage_tabs_message='Saved.')

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_addLink')
    def manage_addLink(self,title, expression, REQUEST=None):
        """
        Add a new link dictionary.
        """
        link_element = {"title":title, "expression":expression}
        link_elements = list(self._links)
        link_elements.append(link_element)
        self._links = tuple(link_elements)
        self._p_changed = 1

        if REQUEST is not None:
            return self.manage_Links(
                self, self.REQUEST, manage_tabs_message='%s added.' % title)

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_deleteLinks')
    def manage_deleteLinks(self, ids=(), REQUEST=None):
        """
        Delete the requested Llink
        """
        ids = list( map( int, ids ) )  # Convert to a list of integers.
        if ids:
            new_link_elements = []
            link_elements = list(self._links)
            for idx in range(len(link_elements)):
                if idx not in ids:
                    new_link_elements.append(link_elements[idx])
            self._links = tuple(new_link_elements)
            self._p_changed = 1
        if REQUEST:
            return self.manage_Links(
                self, self.REQUEST, manage_tabs_message='Link(s) deleted.')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'applySettingsPortalwise')
    def applySettingsPortalwise(self, contentfolder=None, redirect=1):
        """
        XXX Needs to be updated to the new template style !!!

        Apply the settings on portal level to the rest of the portal (width and title and lock)
        """
        zLOG.LOG(self.meta_type, zLOG.INFO, 'Apply settings called. Folder is %s' % str(contentfolder))
        portal_url = getToolByName(self, 'portal_url')
        portal = portal_url.getPortalObject()
        portalcols = self.getLayoutColumnsFor(portal)
        portaltop = self.getLayoutTopbarFor(portal)
        portalmain = self.getMainColumn(portal)
        if not contentfolder:
            contentfolder = portal
        for obj in contentfolder.contentValues():
            if obj.isPrincipiaFolderish and obj.meta_type != 'CMF Collector':
                objcols = self.getLayoutColumnsFor(obj)
                i = 0
                if portalcols:
                    for col in portalcols:
                        objcol = objcols[i]
                        isLocked = 0
                        if col.hasProperty('isLocked') and col.isLocked == 1:
                            isLocked = 1
                        zLOG.LOG(self.meta_type, zLOG.INFO, 'Above col has data:\nTitle %s\nWidth %s\nisLocked %d' % \
                            (col.title, col.width, isLocked))
                        zLOG.LOG(self.meta_type, zLOG.INFO, 'New col has data:\nTitle %s\nWidth %s\nisLocked %s' % \
                            (objcol.title, getattr(objcol, 'width', 'Not set'), str(getattr(objcol, 'isLocked', 'Not set'))))
                        objcol.editColumn(ctype='column', title=col.title, column_width=col.width, column_locked=isLocked, redirect=0)
                        if col == portalmain:
                            mainpath = portal_url.getRelativeContentURL(objcol)
                            self.setMainColumn(mainpath, obj)
                        portal_elems = []
                        local_elems = []
                        for element in self.listLayoutObjectsFor(objcol):
                            if element.meta_type == 'Ghost Element':
                                portal_elems.append(element.getId())
                            else:
                                local_elems.append(element)
                        topelems = self.listLayoutObjectsFor(col)
                        self.deleteLayoutIn(objcol, portal_elems)
                        ecount = 0
                        for elem in self.listLayoutObjectsFor(col):
                            ref_path = elem.absolute_url(relative=1)
                            self.addLayoutTo(columnfolder=objcol,layout_type='Ghost Element', referer_path=ref_path)
                            newlt = None
                            for lt in self.listLayoutObjectsFor(objcol):
                                if lt.meta_type == 'Ghost Element':
                                    if lt.getReferencePath() == ref_path:
                                        newlt = lt
                            lt.setSortOrder(ecount)
                            ecount = ecount + 1
                        for elem in local_elems:
                            elem.setSortOrder(ecount)
                            ecount = ecount + 1
                        i = i + 1
                objtopbar = self.getLayoutTopbarFor(obj, aquire=0)
                portaltopwidth = portaltop.hasProperty('width') and portaltop.width or '0'
                objtopbar.editColumn(ctype='topbar', title=portaltop.title, column_width=portaltopwidth, redirect=0)
                self.applySettingsPortalwise(contentfolder=obj, redirect=0)
        if redirect:
            req = self.REQUEST
            req_url = req.get('HTTP_REFERER')
            has_q = req_url.find('?')
            if has_q != -1:
                req_url, trash = req_url.split('?')
            msg = 'Settings applied to entire portal'
            req.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declareProtected(CMFCorePermissions.ManagePortal, 'resetSubFolders')
    def resetSubFolders(self, contentfolder=None, redirect=1):
        """
        XXX Needs to be updated to the new template style !!!

        Reset sub folders when portal layout is perfect !
        (Will skip columns in LocalColumns)
        """
        portal_url = getToolByName(self, 'portal_url')
        portal = portal_url.getPortalObject()
        portalcols = self.getLayoutColumnsFor(portal)
        portaltop = self.getLayoutTopbarFor(portal)
        portalmain = self.getMainColumn(portal)
        if not contentfolder:
            contentfolder = portal
        for obj in contentfolder.contentValues():
            if obj.isPrincipiaFolderish and obj.meta_type != 'CMF Collector':
                localcols = self.getLocalLayoutColumnsFor(obj, aquire=0)
                zLOG.LOG("resetSubFolders", zLOG.INFO, 'Local columns for obj %s are %s' % (obj.absolute_url(), str(localcols)))
                for subobj in obj.objectValues():
                    if subobj.getId()[:3] == 'clf':
                        if localcols and subobj not in localcols:
                            zLOG.LOG("%s - resetSubFolders" % self.meta_type, zLOG.INFO, 'We have subcols. We are deleting a column with id %s' % subobj.getId())
                            obj.manage_delObjects(subobj.getId())
                        else:
                            zLOG.LOG("%s - resetSubFolders" % self.meta_type, zLOG.INFO, 'We have NONE subcols. We are deleting a column with id %s' % subobj.getId())
                            obj.manage_delObjects(subobj.getId())
                obj.manage_changeProperties(columnList=[], mainColumn='')
                objcols = self.getLayoutColumnsFor(obj)
                objtopbar = self.getLayoutTopbarFor(obj, aquire=0)
                self.resetSubFolders(contentfolder=obj, redirect=0)
        if redirect:
            req = self.REQUEST
            req_url = req.get('HTTP_REFERER')
            has_q = req_url.find('?')
            if has_q != -1:
                req_url, trash = req_url.split('?')
            msg = 'Sub folders is reset'
            req.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declarePublic('getParentContainerFor')
    def getParentContainerFor(self, container):
        """"
        Return the next container upwards in the hierarchy which
        could have a layoutcontainer.
        """
        if not getattr(container, '_isPortalRoot', None):
            item = container
            _notfound = []
            while 1:
                item = getattr(item, 'aq_parent', _notfound)
                if item is _notfound:
                    break
                elif item.isPrincipiaFolderish:
                    break
                elif item.id == 'talkback':
                    break
            if item is not container:
                return item

    #PAGE METHODS

    security.declareProtected(LayoutPermissions.AddLayoutTemplate, 'addLayoutTemplate')
    def addLayoutTemplate(self, title, REQUEST=None):
        """
        Add a new layout template
        """
        portal = getToolByName(self, 'portal_url').getPortalObject()
        newid = getUniqueId()
        portal.invokeFactory(id=newid, type_name='Portal Template')
        template = getattr(portal, newid)
        template.setTitle(title)
        if REQUEST is not None:
            msg = 'Created template %s' % title
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            if REQUEST.has_key('frameable'):
                extra = '&frameable=true'
            else:
                extra = ''
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s%s' % (req_url, msg.replace(' ', '+'), extra))

    security.declareProtected(LayoutPermissions.DeleteLayoutTemplate, 'deleteLayoutTemplate')
    def deleteLayoutTemplate(self, templatepath, REQUEST=None):
        """
        Delete the requested layout template
        """
        try:
            portal = getToolByName(self, 'portal_url').getPortalObject()
            template = portal.restrictedTraverse(templatepath)
            parent = self.getParentContainerFor(template)
            templateid = template.getId()
            templatetitle = template.title
            parent.manage_delObjects(templateid)
            msg = 'Deleted template %s' % templatetitle
        except:
            msg = 'Template %s not found' % templatepath

        if REQUEST is not None:
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            if REQUEST.has_key('frameable'):
                extra = '&frameable=true'
            else:
                extra = ''
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s%s' % (req_url, msg.replace(' ', '+'), extra))

    security.declarePublic('getLayoutTemplates')
    def getLayoutTemplates(self):
        """
        Return a list of all availiable layout templates
        """
        catalog = getToolByName(self, 'portal_catalog')
        brains = catalog(meta_type='Portal Layout Template')
        objs = []
        for brain in brains:
            try:
                ob = brain.getObject()
                objs.append(ob)
            except:
                pass
        return objs

    security.declareProtected(LayoutPermissions.ChangeLayoutTemplate, 'setLayoutTemplateFor')
    def setLayoutTemplateFor(self, container, templatepath, REQUEST=None):
        """
        Set the path to the layout template to use
        """
        if type(container) is type(''):
            container = self.portal_url.getPortalObject().restrictedTraverse(container)
        if not container.hasProperty('layoutTemplate'):
            container.manage_addProperty(id='layoutTemplate', value='', type='string')
        container.manage_changeProperties(layoutTemplate=templatepath)

        if REQUEST is not None:
            msg = 'Layout Template set'
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declareProtected(LayoutPermissions.View, 'getLayoutTemplateFor')
    def getLayoutTemplateFor(self, container, aquire=1):
        """
        Return the template to be used for container
        """
        templatepath = container.getProperty('layoutTemplate')
        portal = getToolByName(self, 'portal_url').getPortalObject()
        template = None
        if not templatepath:
            try:
                objtype = container.Type()
            except:
                objtype = None
            if objtype:
                templatepath = self.getTemplateForType(objtype)
        if templatepath:
            try:
                template = portal.restrictedTraverse(templatepath)
            except:
                if aquire and container.absolute_url() != portal.absolute_url():
                    container = self.getParentContainerFor(container)
                    template = self.getLayoutTemplateFor(container)
                else:
                    templates = self.getLayoutTemplates()
                    if templates:
                        template = templates[0]
                    else:
                        raise AttributeError, 'No templates found on portal !!!'
                
        else:
            if aquire and container.absolute_url() != portal.absolute_url():
                container = self.getParentContainerFor(container)
                template = self.getLayoutTemplateFor(container)
        return template

    #LOCAL COLUMN METHODS

    security.declareProtected(LayoutPermissions.ChangeLayoutElement, 'addLocalLayoutColumn')
    def addLocalLayoutColumn(self, container, REQUEST=None):
        """
        Add a new local layout column in 'container'
        """
        if type(container) is type(''):
            container = self.portal_url.getPortalObject().restrictedTraverse(container)
        columnid = "%s%s" % (self.COLUMN_PREFIX, getUniqueId())
        column = LayoutColumn(columnid)
        container._setObject(columnid, column)
        column = getattr(container, columnid)

        if REQUEST is not None:
            msg = 'Added new column'
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declareProtected(LayoutPermissions.ChangeLayoutElement, 'deleteLocalLayoutColumn')
    def deleteLocalLayoutColumn(self, columnfolder, REQUEST=None):
        """
        Delete the request column folder
        """
        if type(columnfolder) is type(''):
            columnfolder = self.portal_url.getPortalObject().restrictedTraverse(columnfolder)
        container = self.getParentContainerFor(columnfolder)
        if container and container.getId() != columnfolder.getId():
            columns = self.getLocalLayoutColumns(container, aquire=0)
            delcol = None
            for column in columns:
                if column.getId() == columnfolder.getId():
                    delcol = column
                    break
            if delcol:
                coltitle = delcol.title_or_id()
                container._delObject(delcol.getId())
                msg = 'Column %s deleted' % coltitle
            else:
                msg = 'Column %s not found' % columnid

        if REQUEST is not None:
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declarePublic('getLocalLayoutColumns')
    def getLocalLayoutColumns(self, container, aquire=1):
        """
        Return the local layout columns
        """
        columns = []
        if type(container) is type(''):
            container = self.portal_url.getPortalObject().restrictedTraverse(container)
        if not container.isPrincipiaFolderish:
            container = aq_parent(container)
        containerid = container.getId()
        if containerid[:4] == self.COLUMN_PREFIX:
            container = self.getParentContainerFor(container)
        columns = []
        if container.objectValues('Portal Layout Column'):
            columns = list(container.objectValues('Portal Layout Column'))
            columns.sort()
        return columns

    security.declareProtected(CMFCorePermissions.View, 'moveLocalLayoutColumn')
    def moveLocalLayoutColumn(self, container, columnid, direction, REQUEST=None):
        """
        Move the requested column in direction
        """
        swapobj = None
        if type(container) is type(''):
            container = self.portal_url.getPortalObject().restrictedTraverse(container)
        swap1 = getattr(container, columnid)
        columns = self.getLocalLayoutColumns(container)

        if direction == 'up':
            for column in columns:
                if swapobj is None: swapobj = column
                if column.getId() != id:
                    if column.sort_order < swap1.sort_order and column.sort_order > swapobj.sort_order:
                        swapobj = column
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)
        elif direction == 'down':
            for column in columns:
                if swapobj is None: swapobj = column
                if column.getId() != id:
                    if column.sort_order > swap1.sort_order and column.sort_order > swapobj.sort_order:
                        swapobj = column
                        break
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)

        if REQUEST is not None:
            msg = 'Moved column %s' % direction
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))


    #FORMAT METHODS

    security.declarePublic('listFormats')
    def listFormats(self):
        """
        Returns a list of formats
        """
        return self.objectValues('Format Object')

    security.declarePublic('listFormatElements')
    def listFormatElements(self):
        """
        Returns a list of format elements
        """
        return self._format_elements

    security.declarePublic('getFormat')
    def getFormat(self, id):
        """
        Returns the formats object with id 'id'
        """
        try:
            obj = self._getOb(id)
            return obj
        except:
            pass

    security.declarePublic('getFormatElementExpression')
    def getFormatElementExpression(self, name):
        """
        Returns the TALES expression for 'name'
        """
        for element in self._format_elements:
            if element['title'] == name:
                return element['expression']

    security.declarePrivate( '_extractFormatElement' )
    def _extractFormatElement( self, properties, index ):

        """
        Extract a style from the funky form properties.
        Got this from ActionProviderBase.
        """
        title       = str( properties.get( 'title_%d'       % index, '' ) )
        expression  = str( properties.get( 'expression_%d'   % index, '' ) )

        format_element = {
                "title":title
               ,"expression":expression
               }

        return format_element


    #LINK METHODS

    security.declarePublic('listLinkElements')
    def listLinkElements(self):
        """
        Returns a list of link elements
        """
        return self._links

    security.declarePublic('getLinkExpression')
    def getLinkExpression(self, name):
        """
        Return the TALES expression for 'name'
        """
        for element in self._links:
            if element['title'] == name:
                return element['expression']
        return None

    security.declarePrivate('_extractLinkElement')
    def _extractLinkElement(self, properties, index):
        """
        Extract the link from the properties
        """
        title = str(properties.get('title_%d' % index, ''))
        expression = str(properties.get('expression_%d' % index, ''))
        link_element = {'title': title, 'expression': expression}
        return link_element


    #STYLES METHODS

    security.declarePublic('listStyles')
    def listStyles(self):
        """
        Returns the style list
        """
        return self._styles

    security.declarePublic('getStyleByName')
    def getStyleByName(self, name):
        """
        Returns the style dictionary with title 'name' or None if not found
        """
        for style in self._styles:
            if style['title'] == name:
                return style

    security.declarePublic('getStyleLineByName')
    def getStyleLineByName(self, name):
        """
        Returns a css-formatted line compiled from style name
        """
        style = self.getStyleByName(name)
        if style:
            line = ''
            if style.get('font_size', None):
                line = "%s\nfont-size: %s;" % (line, style['font_size'])
            if style.get('font_style', None):
                line = "%s\nfont-style: %s;" % (line, style['font_style'])
            if style.get('font_weight', None):
                line = "%s\nfont-weight: %s;" % (line, style['font_weight'])
            if style.get('font_color', None):
                line = "%s\ncolor: %s;" % (line, style['font_color'])
            if style.get('link_image', None):
                position = style.get('image_position', 'left')
                imagepadding = style.get('image_padding', '15px')
                if position == 'left':
                    padding = "1px 1px 1px %s" % imagepadding
                else:
                    padding = "1px %s 1px 1px" % imagepadding
                line = """%s
                       background-image: url(%s);
                       background-repeat: no-repeat;
                       background-position: %s;
                       padding: %s;
                       display: inline;""" % (line, style['link_image'], position, padding)
            return line

    security.declarePrivate('_testStyleTitleValidity')
    def _testStyleTitleValidity(self, title, index=None):
        """
        Test if the title is already occupied
        """
        if not title:
            raise "ValueError", "An empty title is not allowed"
        idx = 0
        for style in self._styles:
            if style['title'] == title:
                if index is None or index != idx:
                    raise "ValueError", "The title '%s' is already in use" % title
            idx = idx + 1

    security.declarePrivate( '_extractStyle' )
    def _extractStyle( self, properties, index ):

        """
        Extract a style from the funky form properties.
        Got this from ActionProviderBase.
        """
        title       = str( properties.get( 'title_%d'       % index, '' ) )
        font_size   = str( properties.get( 'font_size_%d'   % index, '' ) )
        font_style  = str( properties.get( 'font_style_%d'  % index, '' ) )
        font_weight = str( properties.get( 'font_weight_%d' % index, '' ) )
        font_color  = str( properties.get( 'font_color_%d'  % index, '' ) )
        link_image  = str( properties.get( 'link_image_%d'  % index, '' ) )
        image_position = str( properties.get('image_position_%d' % index, '' ) )
        image_padding = str( properties.get('image_padding_%d' % index, '' ) )

        rgx = re.compile(r'(^_|[^a-zA-Z0-9-_~\,\.])')
        title = re.sub(rgx, "",title)
        self._testStyleTitleValidity(title, index)
        style = {
                "title":title
               ,"font_size":font_size
               ,"font_style":font_style
               ,"font_weight":font_weight
               ,"font_color":font_color
               ,"link_image":link_image
               ,"image_position":image_position
               ,"image_padding":image_padding
               }

        return style


def registerLayoutType(type):
    """
    Registers a new layout type with the tool
    """
    if not type in LayoutTool._layout_types:
        LayoutTool._layout_types.append(type)

InitializeClass( LayoutTool )


#Depricated methods
"""
security.declarePublic('deleteLayoutContainerFor')
    def deleteLayoutColumnFor(self, columnfolder, columnidx=None):

        Delete the request column folder

        portal_url = getToolByName(self, 'portal_url')
        portal = portal_url.getPortalObject()
        if type(columnfolder) is type(''):
            columnfolder = portal.restrictedTraverse(columnfolder)
        container = self.getParentContainerFor(columnfolder)
        if container and container.getId() != columnfolder.getId():
            columnpaths = container.getProperty('columnList')
            columnpath = portal_url.getRelativeContentURL(columnfolder)
            if not columnidx:
                columnidx = columnpaths.index(columnpath)
            columnpaths.remove(columnpath)
            container._delObject(columnfolder.getId())
        container.manage_changeProperties(columnList=columnpaths)
        for obj in container.contentValues():
            if obj.isPrincipiaFolderish and obj.meta_type != 'CMF Collector':
                objcolumnpaths = obj.getProperty('columnList')
                objcolumnpath = objcolumnpaths[columnidx]
                objcolumn = portal.restrictedTraverse(objcolumnpath)
                try:
                    self.deleteLayoutColumnFor(columnfolder=objcolumn, columnidx=columnidx)
                except:
                    zLOG.LOG('LayoutTool', zLOG.INFO, 'Exception in deleteLayoutColumnFor: Type: %s - Value: %s' % \
                   (sys.exc_info()[0], sys.exc_info()[1]))

    security.declareProtected(CMFCorePermissions.ManagePortal, 'addLayoutColumnIn')
    def addLayoutColumnIn(self, container):

        Add a new layout column in 'container'

        if type(container) is type(''):
            container = self.portal_url.getPortalObject().restrictedTraverse(container)
        columnid = "%s%s" % (self.COLUMN_PREFIX, getUniqueId())
        container.manage_addProduct['OFSP'].manage_addFolder(columnid,title="CMF Portal Layouts")
        column = getattr(container, columnid)
        if not container.hasProperty('columnList'):
            container.manage_addProperty(id='columnList', value=[], type='lines')
        columns = container.getProperty('columnList')
        portal_url = getToolByName(self, 'portal_url')
        newpath = portal_url.getRelativeContentURL(column)
        columns.append(newpath)
        container.manage_changeProperties(columnList=columns)
        for obj in container.contentValues():
            if obj.isPrincipiaFolderish and obj.meta_type != 'CMF Collector':
                self.addLayoutColumnIn(obj)

    security.declarePublic('getLayoutColumnsFor')
    def getLayoutColumnsFor(self, container, aquire=0):

        Return the layout containers

        columns = []
        if type(container) is type(''):
            container = self.portal_url.getPortalObject().restrictedTraverse(container)
        if not container.isPrincipiaFolderish:
            container = aq_parent(container)
        containerid = container.getId()
        if containerid[:4] == self.COLUMN_PREFIX:
            container = self.getParentContainerFor(container)
        columnpaths = container.getProperty('columnList', [])
        portal = getToolByName(self, 'portal_url').getPortalObject()
        for path in columnpaths:
            column = portal.restrictedTraverse(path)
            if column.getId()[:4] == self.COLUMN_PREFIX:
                columns.append(column)
        portal = getToolByName(self, 'portal_url').getPortalObject()
        if not columns and container.meta_type != 'Layout Box' and container != portal and not aquire:
            member = getToolByName(self, 'portal_membership').getAuthenticatedMember()
            zLOG.LOG(self.meta_type, zLOG.INFO, 'Are we owner? %d' % member.has_role('Owner', container))
            zLOG.LOG(self.meta_type, zLOG.INFO, 'Are we Manager? %d' % member.has_role('Manager', container))
            if member.has_role('Owner', container) or member.has_role('Manager', container):
                columns = self.initializeSubColumns(container)

        #if not columns and container.meta_type == 'Layout Box' and aquire:
            #parent = container.aq_parent
            #columns = self.getLayoutColumnsFor(parent)

        if not columns and container.meta_type == 'Layout Box':
            parent = container.aq_parent
            columns = self.getLayoutColumnsFor(parent)
        elif not columns and container != portal and aquire:
            parent = container.aq_parent
            columns = self.getLayoutColumnsFor(parent)
        if columns:
            return columns

    security.declarePrivate('initializeSubColumns')
    def initializeSubColumns(self, container):
        If a folder has no columns, we will initialize it with the columns from the above folder
        parent = self.getParentContainerFor(container)
        columns = self.getLayoutColumnsFor(parent, aquire=0)
        zLOG.LOG("%s - init sub columns" % self.meta_type, zLOG.INFO, 'Parent is %s' % \
        parent.absolute_url(relative=1))
        maincol = self.getMainColumn(parent)
        portal_url = getToolByName(self, 'portal_url')
        portal = portal_url.getPortalObject()
        returncols = []
        for column in columns:
            self.addLayoutColumnIn(container)
            colpath = container.getProperty('columnList')[-1:]
            colpath = colpath[0]
            subcolumn = portal.restrictedTraverse(colpath)
            for layout in self.listLayoutObjectsFor(column):
                refererpath = layout.meta_type == 'Ghost Element' and layout.getReferencePath() or layout.absolute_url(relative=1)
                self.addLayoutTo(columnfolder=subcolumn, layout_type='Ghost Element', referer_path=refererpath)
            if column == maincol:
                mainpath = portal_url.getRelativeContentURL(subcolumn)
                self.setMainColumn(mainpath, container)
            if column.hasProperty('isLocked') and column.isLocked:
                isLocked = 1
            else:
                isLocked = 0
            colwidth = getattr(column, 'width', '0')
            subcolumn.editColumn(title=column.title, column_width=colwidth, column_locked=isLocked, redirect=0)
            returncols.append(subcolumn)
        return returncols

security.declarePublic('getMainColumn')
    def getMainColumn(self, obj=None):

        Return the main column (the one containing metal:define-slot="main")

        column = None
        portal = getToolByName(self, 'portal_url').getPortalObject()
        if not obj:
            obj = portal
        cpath = obj.getProperty('mainColumn')
        column = portal.restrictedTraverse(cpath)
        return column

    security.declareProtected(CMFCorePermissions.ManagePortal, 'setMainColumn')
    def setMainColumn(self, cpath, setOnObject=None):

        Set the path to the main column

        portal_url = getToolByName(self, 'portal_url')
        portal = portal_url.getPortalObject()
        if not setOnObject:
            setOnObject = portal
        if not setOnObject.hasProperty('mainColumn'):
            setOnObject.manage_addProperty(id='mainColumn', value='', type='string')
        columnlist = setOnObject.getProperty('columnList')
        #zLOG.LOG(self.meta_type, zLOG.INFO, 'Cpath is %s' % cpath)
        columnobj = portal.restrictedTraverse(cpath)
        columnpath = portal_url.getRelativeContentURL(columnobj)
        columnidx = columnlist.index(columnpath)
        setOnObject.manage_changeProperties(mainColumn=cpath)
        if setOnObject.isPrincipiaFolderish:
            for obj in setOnObject.contentValues():
                if obj.isPrincipiaFolderish and obj.meta_type != 'CMF Collector':
                    try:
                        objcolumnlist = obj.getProperty('columnList')
                        objcolumnpath = objcolumnlist[columnidx]
                        objcolumn = portal.restrictedTraverse(objcolumnpath)
                        newmainpath = portal_url.getRelativeContentURL(objcolumn)
                        self.setMainColumn(cpath=newmainpath, setOnObject=obj)
                    except:
                        zLOG.LOG(self.meta_type, zLOG.INFO, \
                        'Exception in setting main column: %s - %s\n%s' % \
                        (sys.exc_info()[0]
                        , sys.exc_info()[1]
                        , traceback.format_list(traceback.extract_tb(sys.exc_traceback))
                        ))

security.declareProtected(CMFCorePermissions.View, 'addLayoutTo')
    def addLayoutTo(self, columnfolder, layout_type, title=None, referer_path=None):
        if type(columnfolder) is type(''):
            columnfolder = self.portal_url.getPortalObject().restrictedTraverse(columnfolder)
        layout = None
        for lt in self._layout_types:
            if layout_type == lt.meta_type:
                if referer_path:
                    layout = lt(referer_path, title)
                else:
                    layout = lt(title)
                break

        if layout is None:
            # No menu type matched passed in value
            raise NameError, 'Unknown layout Type: %s' % type
        #try:
        isLocalColumn = 0
        columnfolder._setObject(layout.getId(), layout)
        columnpath = self.portal_url.getRelativeContentURL(columnfolder)
        zLOG.LOG(self.meta_type, zLOG.INFO, 'Columnfolder url is %s' % columnfolder.absolute_url())
        columnparent = self.getParentContainerFor(columnfolder)
        zLOG.LOG(self.meta_type, zLOG.INFO, 'Columnparent is %s' % str(columnparent))
        localcols = self.getLocalLayoutColumnsFor(columnparent, aquire=0)
        if localcols:
            if columnfolder in localcols:
                isLocalColumn = 1

        if not isLocalColumn:
            if not referer_path:
                newlt = getattr(columnfolder, layout.getId())
                referer_path = newlt.absolute_url(relative=1)
            self.createGhostObjects(columnfolder, referer_path)

        #except:
            #zLOG.LOG('LayoutTool', zLOG.INFO, 'Exception in addLayoutTo: Type: %s - Value: %s' % \
                   #(sys.exc_info()[0], sys.exc_info()[1]))


    security.declarePrivate('createGhostObjects')
    def createGhostObjects(self, columnfolder, referer_path):

        Create ghost objects for newly added layout in subfolders of parent to columnfolder
        Referer path points to the newly added layout element

        contentfolder = self.getParentContainerFor(columnfolder)
        if contentfolder.id[:4] == self.COLUMN_PREFIX or contentfolder.id[:7] == self.TOPBAR_PREFIX:
            contentfolder = self.getParentContainerFor(contentfolder)
        if columnfolder.getId()[:7] == self.TOPBAR_PREFIX:
            for obj in contentfolder.contentValues():
                if obj.isPrincipiaFolderish and obj.meta_type != 'CMF Collector':
                    topbar = self.getLayoutTopbarFor(obj, aquire=0)
                    self.addLayoutTo(columnfolder=topbar, layout_type='Ghost Element', referer_path=referer_path)
        elif columnfolder.getId()[:4] == self.COLUMN_PREFIX:
            existing_columns = self.getLayoutColumnsFor(contentfolder)
            columnpaths = contentfolder.getProperty('columnList', [])
            portal_url = getToolByName(self, 'portal_url')
            portal = portal_url.getPortalObject()
            columnpath = portal_url.getRelativeContentURL(columnfolder)
            zLOG.LOG("%s - createGhostObject" % self.meta_type, zLOG.INFO, 'Columnpaths are %s' % str(columnpaths))
            zLOG.LOG("%s - createGhostObject" % self.meta_type, zLOG.INFO, 'Columnpath are %s' % columnpath)
            columnidx = columnpath in columnpaths and columnpaths.index(columnpath) or 0
            for obj in contentfolder.contentValues():
                if obj.isPrincipiaFolderish and obj.meta_type != 'CMF Collector':
                    foldercolumns = obj.getProperty('columnList', [])
                    if len(foldercolumns) < len(columnpaths):
                        missingcols = len(columnpaths) - len(foldercolumns)
                        i = 1
                        while i <= missingcols:
                            self.addLayoutColumnIn(obj)
                            i = i + 1
                    addInColumnList = obj.getProperty('columnList', [])
                    addInColumnPath = obj.getProperty('columnList')[columnidx]
                    #zLOG.LOG(self.meta_type, zLOG.INFO, 'Column index is %d' % columnidx)
                    addInColumn = portal.restrictedTraverse(addInColumnPath)
                    self.addLayoutTo(columnfolder=addInColumn, layout_type='Ghost Element', referer_path=referer_path)
        else:
            raise AttributeError, 'Unknown column folder: %s' % columnfolder.absolute_url(relative=1)

    security.declareProtected(CMFCorePermissions.View, 'deleteLayoutIn')
    def deleteLayoutIn(self, columnfolder, layout_id, reference_path=None):
        Delete selected layout(s)
        isLocalColumn = 0
        columnparent = self.getParentContainerFor(columnfolder)
        localcols = self.getLocalLayoutColumnsFor(columnparent, aquire=0)
        if localcols:
            if columnfolder in localcols:
                isLocalColumn = 1

        if type(layout_id) is type(''):
            if not reference_path:
                layoutobj = getattr(columnfolder, layoutid)
                if layoutobj.meta_type == 'Ghost Element':
                    reference_path = layoutobj.getReferencePath()
            if not isLocalColumn:
                self.deleteGhostObjects(columnfolder, reference_path)
            columnfolder._delObject(layout_id)
        elif type(layout_id) in (type(()), type([])):
            for lid in layout_id:
                if not reference_path:
                    layoutobj = getattr(columnfolder, lid)
                    if layoutobj.meta_type == 'Ghost Element':
                        reference_path = layoutobj.getReferencePath()
                    else:
                        reference_path = layoutobj.absolute_url(relative=1)
                if not isLocalColumn:
                    self.deleteGhostObjects(columnfolder, reference_path)
                columnfolder._delObject(lid)

    security.declarePrivate('deleteGhostObjects')
    def deleteGhostObjects(self, columnfolder, reference_path):

        Delete ghost objects for the layout object about to be deleted in subfolders of parent to columnfolder
        Layout obj is the object being deleted

        contentfolder = self.getParentContainerFor(columnfolder)
        if contentfolder.id[:4] == self.COLUMN_PREFIX or contentfolder.id[:7] == self.TOPBAR_PREFIX:
            contentfolder = self.getParentContainerFor(container)
        if columnfolder.getId()[:7] == self.TOPBAR_PREFIX:
            for obj in contentfolder.contentValues():
                if obj.isPrincipiaFolderish and obj.meta_type != 'CMF Collector':
                    topbar = self.getLayoutTopbarFor(obj, aquire=0)
                    if topbar:
                        for element in self.listLayoutObjectsFor(topbar, ctype='topbar'):
                            if element.meta_type == 'Ghost Element' and element.getReferencePath() == reference_path:
                                self.deleteLayoutIn(columnfolder=topbar, layout_id=element.getId(), reference_path=reference_path)
        elif columnfolder.getId()[:4] == self.COLUMN_PREFIX:
            columnpaths = contentfolder.getProperty('columnList', [])
            portal_url = getToolByName(self, 'portal_url')
            portal = portal_url.getPortalObject()
            columnpath = portal_url.getRelativeContentURL(columnfolder)
            columnidx = columnpath in columnpaths and columnpaths.index(columnpath) or 0
            for obj in contentfolder.contentValues():
                if obj.isPrincipiaFolderish and obj.meta_type != 'CMF Collector':
                    foldercolumns = obj.getProperty('columnList', [])
                    zLOG.LOG("%s - delGhostObjs" % self.meta_type, zLOG.INFO, 'Obj is: %s' % obj.absolute_url())
                    zLOG.LOG("%s - delGhostObjs" % self.meta_type, zLOG.INFO, 'Foldercolumns are: %s' % str(foldercolumns))
                    zLOG.LOG("%s - delGhostObjs" % self.meta_type, zLOG.INFO, 'Columnidx are: %d' % columnidx)
                    objcolumnpath = foldercolumns[columnidx]
                    objcolumn = portal.restrictedTraverse(objcolumnpath)
                    for element in self.listLayoutObjectsFor(objcolumn):
                        if element.meta_type == 'Ghost Element' and getattr(element, 'getReferencePath', '') == reference_path:
                            self.deleteLayoutIn(columnfolder=objcolumn, layout_id=element.getId(), reference_path=reference_path)
        else:
            raise AttributeError, 'Unknown column folder: %s' % columnfolder.absolute_url(relative=1)

security.declarePublic('listLayoutObjectsFor')
    def listLayoutObjectsFor(self, column, ctype='column'):

        Returns all the layout item objects created in container.
        If layout merging is enabled any layouts from the parent container
        is appended to the list.
        Container must be layout container

        if ctype == 'column':
            if column.getId()[:4] != self.COLUMN_PREFIX:
                column = self.getLayoutColumnsFor(column)
        elif ctype == 'topbar':
            if column.getId()[:7] != self.TOPBAR_PREFIX:
                column = self.getLayoutTopbarFor(column)
        else:
            raise AttributeError, 'Unknown column type: %s' % ctype

        layouts = []
        if column:
            if column.objectValues():
                layouts = list(column.objectValues(self.listLayoutTypes(filter=0)))
                layouts.sort()
        return layouts

    security.declarePublic('listLayoutObjectsForLanguage')
    def listLayoutObjectsForLanguage(self, container, ctype='column'):

        Returns all layout items created in container
        Uses language to determine what items dhould be returned

        layouts = self.listLayoutObjectsFor(container, ctype)
        try:
            msgcatalog = self.restrictedTraverse(self.getMessageCatalog())
            sel_language = msgcatalog.get_selected_language()
        except:
            sel_language = 'en'
        for layout in layouts:
            lan = layout.getLanguage()
            if lan and lan != sel_language:
                layouts.remove(layout)
        return layouts

        security.declarePublic('getLayoutPropertiesFor')
    def getLayoutPropertiesFor(self, container, aquire=1):

        Return the layout properties for the page design

        props = None
        if not container.isPrincipiaFolderish:
            container = container.aq_parent
        path = container.absolute_url(relative=1)
        pagelayouts = self._page_layouts
        if pagelayouts.has_key(path):
            props = pagelayouts[path]
        portal = getToolByName(self, 'portal_url').getPortalObject()
        if not props and container.meta_type == 'Layout Box' and aquire:
            container = container.aq_parent
            props = self.getLayoutPropertiesFor(container)
        elif not props and container != portal and aquire:
            container = container.aq_parent
            props = self.getLayoutPropertiesFor(container)
        return props

    security.declareProtected(CMFCorePermissions.ManagePortal, 'setLayoutPropertiesFor')
    def setLayoutPropertiesFor(self, container, width, align, spacing, padding):

        Set the layout properties for the page layout

        pagelayouts = self._page_layouts
        path = container.absolute_url(relative=1)
        props = {'width': width, 'align': align, 'cellspacing': spacing, 'cellpadding': padding}
        pagelayouts[path] = props
        self._page_layouts = pagelayouts
        self._p_changed = 1

    security.declareProtected(CMFCorePermissions.View, 'moveLayoutIn')
    def moveLayoutIn(self, columnfolder, layout_id, direction, ctype):

        Move selected layout(s)

        swapobj = None
        swap1 = getattr(columnfolder, layout_id)
        objs = self.listLayoutObjectsFor(columnfolder, ctype)

        if direction == 'up':
            for layout in objs:
                if swapobj is None: swapobj = layout
                if layout.getId() != id:
                    if layout.sort_order < swap1.sort_order and layout.sort_order > swapobj.sort_order:
                        swapobj = layout
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)
        elif direction == 'down':
            for layout in objs:
                if swapobj is None: swapobj = layout
                if layout.getId() != id:
                    if layout.sort_order > swap1.sort_order and layout.sort_order > swapobj.sort_order:
                        swapobj = layout
                        break
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)

    security.declareProtected(CMFCorePermissions.View, 'updateSubColumns')
    def updateSubColumns(self, column, title, width, column_locked=0):

        Update sub columns

        contentfolder = self.getParentContainerFor(column)
        columnpaths = contentfolder.getProperty('columnList', [])
        portal_url = getToolByName(self, 'portal_url')
        portal = portal_url.getPortalObject()
        columnpath = portal_url.getRelativeContentURL(column)
        #zLOG.LOG(self.meta_type, zLOG.INFO, 'Type of columnpath: %s' % type(columnpath))
        #zLOG.LOG(self.meta_type, zLOG.INFO, 'Content of columnpath: %s' % str(columnpath))
        if type(columnpath) is type(''):
            zLOG.LOG("%s - updateSubColumns" % self.meta_type, zLOG.INFO, 'path is string!!!')
            columnpath = string.split(columnpath, '/')
            columnpath = tuple(columnpath)

        cidx = 0
        for cpath in columnpaths:
            if type(cpath) is type(''):
                cpath = string.split(cpath, '/')
                cpath = tuple(cpath)
            #zLOG.LOG("%s - updateSubColumns" % self.meta_type, zLOG.INFO, 'Cpath is %s' % str(cpath))
            if cpath == columnpath:
                columnidx = cidx
                break
            cidx = cidx + 1

        #columnidx = columnpaths.index(str(columnpath))
        for obj in contentfolder.contentValues():
            if obj.isPrincipiaFolderish and obj.meta_type != 'CMF Collector':
                foldercolumns = obj.getProperty('columnList', [])
                try:
                    objcolumnpath = foldercolumns[columnidx]
                    objcolumn = portal.restrictedTraverse(objcolumnpath)
                    objcolumn.editColumn(title=title, column_width=width, column_locked=column_locked, redirect=0)
                except:
                    zLOG.LOG(self.meta_type, zLOG.INFO, \
                        'Exception in finding column: %s - %s\n%s' % \
                        (sys.exc_info()[0]
                        , sys.exc_info()[1]
                        , traceback.format_list(traceback.extract_tb(sys.exc_traceback))
                        ))

    security.declareProtected(CMFCorePermissions.View, 'moveLayoutColumn')
    def moveLayoutColumn(self, container, cidx, direction):

        Move the requested column in direction

        cidx = int(cidx)
        paths = container.getProperty('columnList')
        if direction == 'up':
            if cidx > 0 and cidx < len(paths):
                old = paths[cidx - 1]
                paths[cidx - 1] = paths[cidx]
                paths[cidx] = old
        elif direction == 'down':
            if cidx < len(paths)-1 and cidx >= 0:
                old = paths[cidx + 1]
                paths[cidx + 1] = paths[cidx]
                paths[cidx] = old
        else:
            raise AttributeError, 'Unknown direction: %s' % direction
        container.manage_changeProperties(columnList=paths)

    #TOPBAR METHODS
    security.declarePublic('getLayoutTopbarFor')
    def getLayoutTopbarFor(self, container, aquire=1):

        Return the layout topbar

        topbar = None
        portal = getToolByName(self, 'portal_url').getPortalObject()
        if type(container) is type(''):
            container = portal.restrictedTraverse(container)
        if not container.isPrincipiaFolderish:
            container = aq_parent(container)
        containerid = container.getId()
        if containerid[:7] == self.TOPBAR_PREFIX:
            topbar = container
        else:
            if container.id == 'talkback':
                container = self.getParentContainerFor(container)
            for ob in container.objectValues(['Folder']):
                if ob.getId()[:7] == self.TOPBAR_PREFIX:
                    topbar = ob
                    break

        if not topbar and container.meta_type != 'Layout Box' and not aquire:
            member = getToolByName(self, 'portal_membership').getAuthenticatedMember()
            if member.has_role('Owner', container) or member.has_role('Manager', container):
                topbar = self.initializeSubTopbars(container)

        #zLOG.LOG(self.meta_type, zLOG.INFO, 'Container metatype is %s' % container.meta_type)
        #zLOG.LOG(self.meta_type, zLOG.INFO, 'aquire is %d' % aquire)
        #zLOG.LOG(self.meta_type, zLOG.INFO, 'Topbar is %s' % str(topbar))
        if not topbar and container.meta_type == 'Layout Box':
            parent = container.aq_parent
            topbar = self.getLayoutTopbarFor(parent)
        elif not topbar and container != portal and aquire:
            parent = container.aq_parent
            topbar = self.getLayoutTopbarFor(parent)
        if topbar:
            return topbar

    security.declarePrivate('initializeSubTopbars')
    def initializeSubTopbars(self, container):

        If a folder has no columns, we will initialize it with the columns from the above folder

        parent = self.getParentContainerFor(container)
        if not parent:
            return None
        topbar = self.getLayoutTopbarFor(parent, aquire=0)
        portal_url = getToolByName(self, 'portal_url')
        portal = portal_url.getPortalObject()
        subtop = None
        self.addLayoutTopbarIn(container)
        subtop = self.getLayoutTopbarFor(container, aquire=0)
        for layout in self.listLayoutObjectsFor(topbar, 'topbar'):
            refererpath = layout.meta_type == 'Ghost Element' and layout.getReferencePath() or layout.absolute_url(relative=1)
            self.addLayoutTo(columnfolder=subtop, layout_type='Ghost Element', referer_path=refererpath)
        topwidth = getattr(topbar, 'width', '100%')
        subtop.editColumn(ctype='topbar', title=topbar.title, column_width=topwidth, redirect=0)
        return subtop

    security.declareProtected(CMFCorePermissions.ManageProperties, 'addLayoutTopbarIn')
    def addLayoutTopbarIn(self, container):

        Add a new layout topbar in 'container'

        if type(container) is type(''):
            container = self.portal_url.getPortalObject().restrictedTraverse(container)
        columnid = "%s%s" % (self.TOPBAR_PREFIX, getUniqueId())
        container.manage_addProduct['OFSP'].manage_addFolder(columnid,title="CMF Portal Layout Topbar")
        for obj in container.contentValues():
            if obj.isPrincipiaFolderish and obj.meta_type != 'CMF Collector':
                topbar = self.getLayoutTopbarFor(obj, aquire=0)
                if not topbar:
                    self.addLayoutTopbarIn(obj)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'deleteLayoutTopbarFor')
    def deleteLayoutTopbarFor(self, topbarfolder, deleteChilds=0):

        Delete the requested top bar

        if type(topbarfolder) is type(''):
            topbarfolder = self.portal_url.getPortalObject().restrictedTraverse(topbarfolder)
        container = self.getParentContainerFor(topbarfolder)
        if container and container.getId() != topbarfolder.getId():
            container._delObject(topbarfolder.getId())

        if deleteChilds:
            for obj in container.contentValues():
                if obj.isPrincipiaFolderish and obj.meta_type != 'CMF Collector':
                    topbar = self.getLayoutTopbarFor(obj, aquire=0)
                    if topbar:
                        self.deleteLayoutTopbarFor(topbar, deleteChilds=1)


"""
