##############################################################################
#    Copyright (C) 2003  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import string
from OFS.Folder import Folder
from AccessControl import ClassSecurityInfo

from Products.CMFCore.utils import getToolByName

import LayoutPermissions

class LayoutTopbar(Folder):
    """
    A Layout Column is used to put layout elements in
    """

    meta_type = 'Portal Layout Topbar'
    security = ClassSecurityInfo()

    def __init__(self, id, title=''):
        """
        Construct new layout column
        """
        self.id = id
        self.title = title
        self.width = '100%'

    security.declareProtected(LayoutPermissions.ChangeLayoutTopbar, 'edit')
    def edit(self, title=None, width=None, REQUEST=None):
        """
        Edit the topbar settings
        """
        if title is not None:
            self.title = title

        if width is not None:
            self.width = width

        if REQUEST is not None:
            msg = 'Saved topbar settings'
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declareProtected(LayoutPermissions.AddLayoutElement, 'addLayout')
    def addLayout(self, layout_type, layout_title=None, REQUEST=None):
        """
        Add a new layout element to the column
        """
        layout = None
        ltool = getToolByName(self, 'portal_layout')
        for lt in ltool.listLayoutClasses():
            if layout_type == lt.meta_type:
                layout = lt(layout_title)
                break

        if layout is None:
            # No layout type matched passed in value
            raise NameError, 'Unknown layout type: %s' % layout_type

        self._setObject(layout.getId(), layout)
        if REQUEST is not None:
            msg = 'Added new %s' % layout_type
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declareProtected(LayoutPermissions.DeleteLayoutElement, 'deleteLayout')
    def deleteLayout(self, layoutids, REQUEST=None):
        """
        Delete the requested layout
        """
        msg = []
        for layoutid in layoutids:
            if hasattr(self, layoutid):
                layout = getattr(self, layoutid)
                layouttitle = layout.title_or_id()
                self.manage_delObjects(layoutid)
                msg.append('Deleted layout %s' % layouttitle)
            else:
                msg.append('Layout with id %s not found' % layoutid)

        msg = string.join(msg, ', ')

        if REQUEST is not None:
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declareProtected(LayoutPermissions.ChangeLayoutTopbar, 'moveLayout')
    def moveLayout(self, layout_id, direction, REQUEST=None):
        """
        Move selected layout
        """
        swapobj = None
        swap1 = getattr(self, layout_id)
        objs = self.getLayoutElements()

        if direction == 'up':
            for layout in objs:
                if swapobj is None: swapobj = layout
                if layout.getId() != id:
                    if layout.sort_order < swap1.sort_order and layout.sort_order > swapobj.sort_order:
                        swapobj = layout
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)
        elif direction == 'down':
            for layout in objs:
                if swapobj is None: swapobj = layout
                if layout.getId() != id:
                    if layout.sort_order > swap1.sort_order and layout.sort_order > swapobj.sort_order:
                        swapobj = layout
                        break
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)

        if REQUEST is not None:
            msg = 'Moved layout %s' % direction
            req_url = REQUEST.get('HTTP_REFERER')
            if req_url.find('?') != -1:
                req_url, trash = req_url.split('?')
            REQUEST.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declareProtected(LayoutPermissions.View, 'getLayoutElements')
    def getLayoutElements(self):
        """
        Return a sorted list of layout elements
        """
        layouts = []
        if self.objectValues():
            ltool = getToolByName(self, 'portal_layout')
            layouts = list(self.objectValues(ltool.listLayoutTypes(filter=0)))
            layouts.sort()
        return layouts

    security.declareProtected(LayoutPermissions.View, 'getLayoutElementsForLanguage')
    def getLayoutElementsForLanguage(self):
        """
        Returns a sorted list of layout elements
        Uses language to determine what elements should be returned
        """
        layouts = self.getLayoutElements()
        ltool = getToolByName(self, 'portal_layout')
        try:
            msgcatalog = ltool.restrictedTraverse(ltool.getMessageCatalog())
            sel_language = msgcatalog.get_selected_language()
        except:
            sel_language = 'en'
        for layout in layouts:
            lan = layout.getLanguage()
            if lan and lan != sel_language:
                layouts.remove(layout)
        return layouts