##############################################################################
#    Copyright (C) 2003  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG, sys
from Globals import InitializeClass, package_home
from AccessControl import getSecurityManager, ClassSecurityInfo
from OFS.Folder import Folder
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from DateTime import DateTime
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId
from Products.CMFPortalLayout.LayoutColumn import LayoutColumn
from Products.CMFPortalLayout import LayoutPermissions

class LayoutBox(Folder, LayoutBase):
    """
    A menu containing layout columns and objects
    """
    _edit_method = 'layoutbox_edit_form'
    _view_method = 'layoutbox_view'
    _standalone_method = 'layoutbox_standalone_view'

    security = ClassSecurityInfo()
    type = 'Layout Element'
    meta_type = 'Layout Element'

    def __init__(self, title=''):
        """
        Create a new layout box object.
        """
        self.id = getUniqueId()
        self.setTitle(title)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        self.setTitle(REQUEST.get('boxtitle', None))

        msg = 'Saved Layout Box'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/layoutbox_edit_form" % self.absolute_url()

        req.RESPONSE.redirect('%s?portal_status_message=%s' % (returnurl, msg.replace(' ', '+')))

    security.declareProtected(LayoutPermissions.AddLayoutColumn, 'addColumn')
    def addLayoutColumn(self, REQUEST=None):
        """
        Add a new layout column
        """
        column = LayoutColumn(getUniqueId())
        self._setObject(column.getId(), column)
        if REQUEST is not None:
            req = REQUEST
            msg = 'Added column'
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = req.get('HTTP_REFERER')
                if returnurl.find('?') != -1:
                    returnurl, trash = returnurl.split('?')
                #returnurl = "%s/layoutbox_edit_form" % self.absolute_url()
            req.RESPONSE.redirect('%s?portal_status_message=%s' % (returnurl, msg.replace(' ', '+')))

    security.declareProtected(LayoutPermissions.DeleteLayoutColumn, 'deleteLayoutColumn')
    def deleteLayoutColumn(self, columnid, REQUEST=None):
        """
        Delete the requested layout column
        """
        if hasattr(self, columnid):
            column = getattr(self, columnid)
            if column.meta_type == 'Portal Layout Main Column':
                msg = 'You are not allowed to delete the main column'
            else:
                columntitle = column.title_or_id()
                self.manage_delObjects(columnid)
                msg = 'Deleted column %s' % columntitle
        else:
            msg = 'Column with id %s not found' % columnid

        if REQUEST is not None:
            req = REQUEST
            parent = self.getParent()
            msg = 'Deleted column'
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = req.get('HTTP_REFERER')
                if returnurl.find('?') != -1:
                    returnurl, trash = returnurl.split('?')
                #returnurl = "%s/layoutbox_edit_form" % self.absolute_url()
            req.RESPONSE.redirect('%s?portal_status_message=%s' % (returnurl, msg.replace(' ', '+')))

    security.declareProtected(LayoutPermissions.MoveLayoutColumn, 'moveLayoutColumn')
    def moveLayoutColumn(self, columnid, direction, REQUEST=None):
        """
        Move the requested layout column in direction
        """
        swapobj = None
        swap1 = getattr(self, columnid)
        columns = self.getLayoutColumns()

        if direction == 'up':
            for column in columns:
                if swapobj is None: swapobj = column
                if column.getId() != id:
                    if column.sort_order < swap1.sort_order and column.sort_order > swapobj.sort_order:
                        swapobj = column
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)
        elif direction == 'down':
            for column in columns:
                if swapobj is None: swapobj = column
                if column.getId() != id:
                    if column.sort_order > swap1.sort_order and column.sort_order > swapobj.sort_order:
                        swapobj = column
                        break
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)

        if REQUEST is not None:
            req = REQUEST
            parent = self.getParent()
            msg = 'Moved column %s' % direction
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = req.get('HTTP_REFERER')
                if returnurl.find('?') != -1:
                    returnurl, trash = returnurl.split('?')
                #returnurl = "%s/layoutbox_edit_form" % self.absolute_url()
            req.RESPONSE.redirect('%s?portal_status_message=%s' % (returnurl, msg.replace(' ', '+')))

    security.declareProtected(LayoutPermissions.View, 'getLayoutColumns')
    def getLayoutColumns(self):
        """
        Get the layout columns sorted for this template
        """
        columns = []
        if self.objectValues(['Portal Layout Column', 'Portal Layout Main Column']):
            columns = list(self.objectValues(['Portal Layout Column', 'Portal Layout Main Column']))
            columns.sort()
        return columns

    security.declareProtected(LayoutPermissions.View, 'getMainColumn')
    def getMainColumn(self):
        """
        Return the column that is main
        """
        maincols = self.objectValues('Portal Layout Main Column')
        if maincols:
            maincol = maincols[0]
        else:
            cols = self.getLayoutColumns()
            if cols:
                maincol = cols[0]
            else:
                maincol = None
        return maincol

    security.declareProtected(LayoutPermissions.View, 'getMainColumnId')
    def getMainColumnId(self):
        """
        Return the id of the column that is main
        """
        maincol = self.getMainColumn()
        colid = None
        if maincol:
            colid = maincol.getId()
        return colid

registerLayoutType(LayoutBox)