##############################################################################
#    Copyright (C) 2002-03  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG
import sys, traceback
from Globals import InitializeClass, package_home
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import _getViewFor
from DateTime import DateTime
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId, getIdFromPath

class ListMenu(LayoutBase):
    """
    An object containing the information for creating a list box.
    """
    _itemCount = 10
    _subjects = []
    _types = []
    _ownSubjects = []
    _sort_criteria ='Date'
    _sort_order ='descending'
    _search_in = 'folder'
    _edit_method = 'listmenu_edit_form'
    _view_method = 'listmenu_view'
    _standalone_method = 'listmenu_standalone_view'
    _format = ''
    _subjectSelector = []
    _useSelector = 0
    _useLangInSearch = 0
    _useBrowseButtons = 1
    _searchGroups = []

    security = ClassSecurityInfo()
    meta_type = 'List Box'

    def __init__(self, title=''): #, itemCount=0, subjects=[], types_dict={}, sort_criteria='', sort_order=None, format_types=[], language=''):
        """
        Create a new list box object.
        """
        self.id = getUniqueId()
        self.setTitle(title)
        self._itemCount = 10
        self._subjects = []
        self._types = []
        self._sort_criteria ='Date'
        self._sort_order ='descending'
        self._search_in = 'folder'
        self._format = ''
        self._subjectSelector = []
        self._useSelector = 0
        self._useLangInSearch = 0
        self._useBrowseButtons = 1
        self._ownSubjects = []
        self._searchGroups = []

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        self.setTitle(REQUEST.get('title', None))
        self.setItemCount(REQUEST.get('itemCount', None))
        self.setSubjects(REQUEST.get('subjects', None))
        self.setOwnSubjects(REQUEST.get('own_subjects', None))
        self.setTypes(REQUEST.get('types', None))
        self.setSearch(REQUEST.get('search_in', None))
        self.setSortCriteria(REQUEST.get('sort_criteria', None))
        self.setCatalogSortOrder(REQUEST.get('sort_order', None))
        self.setFormat(REQUEST.get('format', None))
        self.setSubjectSelector(REQUEST.get('selectSubjects', None))
        self.setUseSelector(REQUEST.get('useSelector', None))
        self.setLanguage(REQUEST.get('item_language', None))
        self.setSearchLanguage(REQUEST.get('use_language', None))
        self.setShowupFor(REQUEST.get('showupForType', None))
        self.setUseBrowseButtons(REQUEST.get('useBrowseButtons', None))
        self.setUserOrGroupSearch(REQUEST.get('usersearch', None))
        self.setGroupSearch(REQUEST.get('groupsearch', None))

        msg = 'Saved list menu'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/listmenu_edit_form" % self.absolute_url()
        req.RESPONSE.redirect('%s?portal_status_message=%s' % (returnurl, msg.replace(' ', '+')))

    security.declarePublic('getItemCount')
    def getItemCount(self):
        """
        Return the item count for the menu object.
        (How many items will show up in the lists).
        """
        return int(self._itemCount)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setItemCount')
    def setItemCount(self, itemCount=None):
        """
        Update the item count of the menu object.
        Defines how many items will show up in the day list.
        """
        if itemCount is not None:
            try:
                self._itemCount = int(itemCount)
            except:
                self._itemCount = 0

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setUseBrowseButtons')
    def setUseBrowseButtons(self, useButtons=None):
        """
        Decide whether to use browse buttons on list or not
        """
        if useButtons is not None:
            self._useBrowseButtons = 1
        else:
            self._useBrowseButtons = 0
            
    security.declarePublic('useBrowseButtons')
    def useBrowseButtons(self):
        """
        Should we insert browse buttons?
        """
        return int(self._useBrowseButtons)

    security.declarePublic('getFormat')
    def getFormat(self):
        """
        Return the format
        """
        if not hasattr(self, '_format'):
            self._format = ''
        return self._format

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setFormat')
    def setFormat(self, format=None):
        """
        Update the item count of the menu object.
        Defines how many items will show up in the day list.
        """
        if format is not None:
            self._format = format

    security.declarePublic('getSubjects')
    def getSubjects(self):
        """
        Returns the subjects for the listmenu object.
        """
        return self._subjects and self._subjects or []

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setSubjects')
    def setSubjects(self, subjects=None):
        """

        """
        if subjects is not None:
            if type(subjects) is type(''):
                subjects = subjects.split('\n')
            new_subjects = []
            for subject in subjects:
                s = subject.strip()
                if s:
                    new_subjects.append(s)
            self._subjects = new_subjects
            
    security.declarePublic('getOwnSubjects')
    def getOwnSubjects(self):
        """
        Returns the own defined subjects
        """
        return self._ownSubjects and self._ownSubjects or None

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setOwnSubjects')
    def setOwnSubjects(self, subjects=None):
        """
        Set the own defined subjects
        """
        if subjects is not None:
            if type(subjects) is type(''):
                subjects = subjects.split('\n')
            new_subjects = []
            for subject in subjects:
                s = subject.strip()
                if s:
                    new_subjects.append(s)
            self._ownSubjects = new_subjects

    security.declarePublic('getAllSubjects')
    def getAllSubjects(self):
        """
        Return both the portal subjects and the own defined subjects
        """
        subjects = []
        if self.getOwnSubjects():
            for sub in self.getOwnSubjects():
                subjects.append(sub)
        if self.getSubjects():
            for sub in self.getSubjects():
                subjects.append(sub)
        return subjects

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setSubjectSelector')
    def setSubjectSelector(self, subjects=None):
        """
        Set the subjects that should be listed as sort options
        """
        if subjects is not None:
            if type(subjects) is type(''):
                subjects = subkects.split('\n')
            new_subjects = []
            for subject in subjects:
                s = subject.strip()
                if s:
                    new_subjects.append(s)
            self._subjectSelector = new_subjects

    security.declarePublic('getSubjectSelector')
    def getSubjectSelector(self):
        """
        Return the list of subjects to use in a sort list
        """
        return self._subjectSelector and self._subjectSelector or []

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setUseSelector')
    def setUseSelector(self, use_selector=None):
        """
        Set a boolean indicating whether to use the subject selector or not
        """
        use = 0
        if use_selector is not None:
            use = 1
        self._useSelector = use

    security.declarePublic('useSubjectSelector')
    def useSubjectSelector(self):
        """
        Return a boolean indication whether to use the subject selector or not
        """
        return self._useSelector and self._useSelector or 0

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setSearch')
    def setSearch(self, search_in=None):
        """
        Update where to search.
        """
        if search_in is not None:
            self._search_in = search_in

    security.declarePublic('getSearch')
    def getSearch(self):
        """
        Returns a string saying where to search ('folder','subfolders','site').
        """
        return self._search_in

    security.declarePublic('getTypes')
    def getTypes(self):
        """
        Return the types selected for the menu object.
        Contains a list of the portal types that should turn up in the menu.
        """
        return self._types

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setTypes')
    def setTypes(self, types=None):
        """
        Update the portal types that will turn up in the calendar box.
        """
        if types is not None:
            self._types = types

    security.declarePublic('getSortCriteria')
    def getSortCriteria(self):
        """
        Returns the sort criteria used for this calendarbox object.
        """
        return self._sort_criteria

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setSortCriteria')
    def setSortCriteria(self, sort_criteria=None):
        """
        Update the sort criteria used for this calendarbox object.
        """
        if sort_criteria is not None:
            self._sort_criteria = sort_criteria

    security.declarePublic('getCatalogSortOrder')
    def getCatalogSortOrder(self):
        """
        Returns the sort order used for this object.
        """
        return self._sort_order

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setCatalogSortOrder')
    def setCatalogSortOrder(self, sort_order=None):
        """
        Update the sort order used for this object.
        """
        if sort_order is not None:
            self._sort_order = sort_order

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setSearchLanguage')
    def setSearchLanguage(self, use_language=None):
        """
        Determine whether the language settings should be used in searches
        """
        if use_language is not None:
            self._useLangInSearch = 1
        else:
            self._useLangInSearch = 0

    security.declarePublic('useLanguageInSearch')
    def useLanguageInSearch(self):
        """
        Return a boolean indication whether the language setting should be used in searches
        """
        return self._useLangInSearch
    
    security.declareProtected(CMFCorePermissions.ManageProperties, 'setGroupSearch')
    def setGroupSearch(self, groupsearch=None):
        """
        Set the list of groupids to include in a search
        """
        if groupsearch is not None:
            self._searchGroups = groupsearch
            
    security.declarePublic('getGroupSearch')
    def getGroupSearch(self):
        """
        Return the ids of the groups to include in a search
        """
        return self._searchGroups

    security.declarePublic('listItems')
    def listItems(self, REQUEST=None):
        """
        Return the items for the menu
        """
        result = None
        searchpars = {}
        search = self.getSearch()
        if search in ('folder','subfolders'):
            obj = REQUEST.get('object')
            if not obj.isPrincipiaFolderish:
                obj = obj.aq_parent
            searchpars['path'] = '/'.join(obj.getPhysicalPath())
            #path = self.getParent().getPhysicalPath()
            #if path:
            #    searchpars['path'] = string.join(path, '/')
        if self.getTypes():
            searchpars['Type'] = self.getTypes()
        else:
            searchpars['Type'] = self.portal_types.objectIds()
        reqvar = "selectedSubject_%s" % self.id
        if REQUEST.has_key(reqvar) and REQUEST[reqvar] != 'all':
            searchpars['Subject'] = REQUEST[reqvar]
        elif self.getAllSubjects():
            searchpars['Subject'] = self.getAllSubjects()
        if self.getCatalogSortOrder():
            searchpars['sort_order'] = self.getCatalogSortOrder()
        searchpars['sort_on'] = self.getSortCriteria()
        
        mtool = getToolByName(self, 'portal_membership')
        if self.getGroupSearch():
            creators = []
            for groupid in self.getGroupSearch():
                creators = creators + list(mtool.getGroupMembers(groupid))
            searchpars['Creator'] = creators

        if self.getUserOrGroupSearch():
            obj = REQUEST.get('object', None)
            if obj:
                creators = searchpars.get('Creator', [])
                usersearch = self.getUserOrGroupSearch()
                if usersearch == 'page_owner':
                    creators.append(obj.Creator())
                elif usersearch == 'page_owner_group':
                    pri_group = mtool.getPrimaryGroupForMember(obj.Creator())
                    if pri_group:
                        gmembers = mtool.getGroupMembers(pri_group)
                        creators = creators + list(gmembers)
                elif usersearch == 'loggedin_member':
                    creators.append(mtool.getAuthenticatedMember().getUserName())
                elif usersearch == 'loggedin_member_group':
                    if not mtool.isAnonymousUser():
                        pri_group = mtool.getPrimaryGroupForMember()
                        if pri_group:
                            gmembers = mtool.getGroupMembers(pri_group)
                            creators = creators + list(gmembers)
                if creators:
                    searchpars['Creator'] = creators
        
        if searchpars.has_key('Creator'):
            creators = searchpars['Creator']
            keeplist = []
            for creator in creators:
                if creator not in keeplist:
                    keeplist.append(creator)
            searchpars['Creator'] = keeplist

        if self.useLanguageInSearch():
            mc = self.getMessageCatalog()
            if mc and mc.get_selected_language():
                searchpars['Language'] = mc.get_selected_language()

        #zLOG.LOG('List Menu - list items', zLOG.INFO, 'Search pars is %s' % str(searchpars))

        #if self.getSortOrder():
        #    searchpars['sort_order'] = 'reverse'

        try:
            catalog = getToolByName(self, 'portal_catalog')
            result = catalog(searchpars)

            if search == 'folder':
                # this is hackish but we have to remove the items that are not directly in this folder
                res = []
                for brain in result:
                    if "%s/%s" % (searchpars['path'],getIdFromPath(brain.getPath())) == brain.getPath():
                        res.append(brain)
                result = res

            #if self.getItemCount():
                #result = result[:int(self.getItemCount())]
            items = []
            #zLOG.LOG('List Menu - list items', zLOG.INFO, 'Search result is %s' % str(result))
            return result
        except:
            zLOG.LOG(self.meta_type, zLOG.ERROR, 'Error in catalog search %s - %s\n%s' \
                % (sys.exc_info()[0]
                 , sys.exc_info()[1]
                 , traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
            return []

registerLayoutType(ListMenu)
