##############################################################################
#    Copyright (C) 2003  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

from AccessControl import ClassSecurityInfo
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId
from Products.DRM2 import PPVTool

class PPVEarnedBox(LayoutBase):
    """
    A menu containing poortal content
    """
    _edit_method = 'ppvearnedbox_edit_form'
    _view_method = 'ppvearnedbox_view'
    _style = None

    security = ClassSecurityInfo()
    type = 'PPV Earned Box'
    meta_type = 'PPV Earned Box'

    def __init__(self, title=''):
        """
        Create a new ppv time object.
        """
        self.id = getUniqueId()
        self.setTitle(title)
        self._style = None

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        self.setTitle(REQUEST.get('title', None))
        self.setLanguage(REQUEST.get('item_language', None))
        self.setRenderTitle(REQUEST.get('render_title', None))
        self.setStyle(REQUEST.get('style', None))

        msg = 'Saved PPV Earned Box'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/ppvearnedbox_edit_form" % self.absolute_url()
        req.RESPONSE.redirect('%s?portal_status_message=%s' % (returnurl, msg.replace(' ', '+')))

    security.declarePublic('getStyle')
    def getStyle(self):
        """
        Return the name style in which the menu entries should be rendered.
        """
        return self._style

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setStyle')
    def setStyle(self, style=None):
        """
        Set the name of the style in which the menu entries should be rendered.
        """
        if style is not None:
            self._style = style

    security.declarePublic('getEarnings')
    def getEarnings(self):
        """
        Get the earnings of the logged-in member
        """
        mtool = getToolByName(self, 'portal_membership')
        username = mtool.getAuthenticatedMember().getUserName()
        ppvtool = getToolByName(self, 'portal_ppv')
        earningobj = None
        owners = ppvtool.objectValues('DRM Owner Party Object')
        for party in owners:
            if party.name == username:
                earningobj = party
                break
            
        if earningobj:
            return earningobj.earnedMoney
        return None
        

registerLayoutType(PPVEarnedBox)