##############################################################################
#    Copyright (C) 2002-03  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG, sys
from Globals import InitializeClass, package_home
from AccessControl import getSecurityManager, ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from DateTime import DateTime
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId

#Page Template stuff
from TAL.TALDefs import TALESError
from Products.PageTemplates import PageTemplate
from Products.PageTemplates.ZopePageTemplate import SecureModuleImporter
from Acquisition import Implicit, aq_parent, aq_inner
from Persistence import Persistent
from Products.CMFDefault.utils import *
from ExtensionClass import Base

class ElementPageTemplate(PageTemplate.PageTemplate, Implicit, Persistent):
    def pt_getContext(self):
        root = self.getPhysicalRoot()
        template = self #aq_parent(self)
        here = aq_parent(template)
        c = {'template': template,
             'options': {},
             'nothing': None,
             'request': None,
             'root': root,
             'request': getattr(root, 'REQUEST', None),
             'modules': SecureModuleImporter,
             'here': here,
             'container': aq_inner(here),
             }
        return c

    def __setstate__(self, state):
        ElementPageTemplate.inheritedAttribute('__setstate__')(self, state)
        self._cook()

class Macros(Base):
    """
    Evaluate to the macros collection of the contained page template.
    """
    def __of__( self, doc):
        return doc.get_pt().macros

class TALElement(LayoutBase):
    """
    A menu containing poortal content
    """
    _contentpaths = []
    _edit_method = 'talelement_edit_form'
    _view_method = 'talelement_view'

    security = ClassSecurityInfo()
    type = 'TAL Element'
    meta_type = 'TAL Element'

    def __init__(self, title=''):
        """
        Create a new content menu object.
        """
        self.id = getUniqueId()
        self.setTitle(title)
        self._pt = ElementPageTemplate()

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        self.setTitle(REQUEST.get('title', None))
        self.setLanguage(REQUEST.get('item_language', None))
        text = REQUEST.get('text', '')
        text = bodyfinder(text)
        self._edit(text)

        parent = self.getParent()
        msg = 'Saved TAL element'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/talelement_edit_form" % self.absolute_url()
        req.RESPONSE.redirect('%s?portal_status_message=%s' % (returnurl, msg.replace(' ', '+')))
        
    def get_pt(self):
        """
            return wrapped PageTemplate object.
        """
        return self._pt.__of__(self)

    def _edit(self, text):
        """
          Edit the document
        """
        pt = self.get_pt()
        pt.write(text)
        
    def CookedBody(self):
        """
           Return a rendering of the Dynamic Document.
        """
        security = getSecurityManager()
        security.addContext(self)
        try:
            try:
                result = self.get_pt().pt_render(
                    extra_context={'user': security.getUser()})
            except TALESError, err:
                if err.type == 'Unauthorized':
                    raise err.type, err.value, err.traceback
                raise
            return result
        finally:
            security.removeContext(self)

    def EditableBody(self):
        """
          The editable body of text.  This is the raw structured text
          or HTML.
        """
        return self.get_pt().read()

    def Format(self):
        """
            Returns a content-type style format of the underlying source
        """
        return self.get_pt().content_type

    def setFormat(self, value):
        """
           Ignore the value, as we only support text/html currently
        """
        self.get_pt().content_type = 'text/html'
        #setFormat = WorkflowAction(setFormat)

    security.declarePublic('getContentPaths')
    def getContentPaths(self):
        """
        Return the content path list
        """
        return self._contentpaths

registerLayoutType(TALElement)
