##############################################################################
#    Copyright (C) 2003  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG
from Globals import InitializeClass, package_home
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import _getViewFor
from DateTime import DateTime
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId, getIdFromPath

class UserGroupList(LayoutBase):
    """
    An object containing the information for creating a list box.
    """
    _itemCount = 10
    _edit_method = 'usergrouplist_edit_form'
    _view_method = 'usergrouplist_view'
    _standalone_method = 'usergrouplist_standalone_view'
    _format = ''
    _useBrowseButtons = 1
    _groupid = None

    security = ClassSecurityInfo()
    meta_type = 'User Group Box'

    def __init__(self, title=''):
        """
        Create a new user group list object.
        """
        self.id = getUniqueId()
        self.setTitle(title)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        self.setTitle(REQUEST.get('title', None))
        self.setItemCount(REQUEST.get('itemCount', None))
        self.setFormat(REQUEST.get('format', None))
        self.setUseBrowseButtons(REQUEST.get('useBrowseButtons', None))
        self.setGroupId(REQUEST.get('groupid', None))

        msg = 'Saved user group list'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/usergrouplist_edit_form" % self.absolute_url()
        req.RESPONSE.redirect('%s?portal_status_message=%s' % (returnurl, msg.replace(' ', '+')))

    security.declarePublic('getGroupId')
    def getGroupId(self):
        """
        Return the group id
        """
        return self._groupid

    security.declarePublic('setGroupId')
    def setGroupId(self, groupid=None):
        """
        Set the group id
        """
        if groupid is not None:
            self._groupid = groupid

    security.declarePublic('getItemCount')
    def getItemCount(self):
        """
        Return the item count for the menu object.
        (How many items will show up in the lists).
        """
        return int(self._itemCount)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setItemCount')
    def setItemCount(self, itemCount=None):
        """
        Update the item count of the menu object.
        Defines how many items will show up in the day list.
        """
        if itemCount is not None:
            try:
                self._itemCount = int(itemCount)
            except:
                self._itemCount = 0

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setUseBrowseButtons')
    def setUseBrowseButtons(self, useButtons=None):
        """
        Decide whether to use browse buttons on list or not
        """
        if useButtons is not None:
            self._useBrowseButtons = 1
        else:
            self._useBrowseButtons = 0

    security.declarePublic('useBrowseButtons')
    def useBrowseButtons(self):
        """
        Should we insert browse buttons?
        """
        return int(self._useBrowseButtons)

    security.declarePublic('getFormat')
    def getFormat(self):
        """
        Return the format
        """
        if not hasattr(self, '_format'):
            self._format = ''
        return self._format

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setFormat')
    def setFormat(self, format=None):
        """
        Update the item count of the menu object.
        Defines how many items will show up in the day list.
        """
        if format is not None:
            self._format = format

    security.declarePublic('listItems')
    def listItems(self, REQUEST=None):
        """
        Return the items for the list
        """
        result = []
        catalog = getToolByName(self, 'portal_catalog')
        memtool = getToolByName(self, 'portal_membership')
        if self.getGroupId():
            groupmembers = memtool.getGroupMembers(self.getGroupId())
            for user in groupmembers:
                #zLOG.LOG(self.meta_type, zLOG.INFO, 'Finding profile for %s' % user)
                #username = user.getUserName()
                brains = catalog(Type='Member Profile', Creator=user)
                if brains:
                    for brain in brains:
                        result.append(brain)
                #else:
                #zLOG.LOG(self.meta_type, zLOG.INFO, 'Did not find profile for %s' % user)
        return result

registerLayoutType(UserGroupList)
